@@ -11,6 +11,10 @@ from coherence import log
 
 import coherence.extern.louie as louie
 
+from coherence.upnp.core.utils import getPage
+from coherence.extern.et import parse_xml
+
+
 class Backend(log.Loggable,Plugin):
 
     """ the base class for all backends
@@ -40,10 +44,21 @@ class Backend(log.Loggable,Plugin):
         self.server = server # the UPnP device that's hosting that backend
 
         """ do whatever is necessary with the stuff we can
-            extract from the config dict
-            and send out the signal when ready
+            extract from the config dict,
+            connect maybe to an external data-source and
+            start up the backend
+            after that's done, tell Coherence about it
         """
-        #louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
+        self.init_completed()
+
+    def init_completed(self, *args, **kwargs):
+        """ inform Coherence that this backend is ready for
+            announcement
+            this method just accepts any form of arguments
+            as we don't under which circumstances it is called
+        """
+        louie.send('Coherence.UPnP.Backend.init_completed',
+                None, backend=self)
 
     def upnp_init(self):
         """ this method gets called after the device is fired,
@@ -106,7 +121,7 @@ class BackendStore(Backend):
 
 
     def _get_all_items(self,id):
-        """ a helper method for to get all items as a response
+        """ a helper method to get all items as a response
             to some XBox 360 UPnP Search action
             probably never be used as the backend will overwrite
             the wmc_mapping with more appropriate methods
@@ -157,7 +172,6 @@ class BackendStore(Backend):
 
         return None
 
-
 class BackendItem(log.Loggable):
 
     """ the base class for all MediaServer backend items
@@ -256,3 +270,33 @@ class BackendItem(log.Loggable):
             an albumArtURI property that does point back to us
         """
         return self.cover
+
+
+class BackendRssMixin:
+
+    def update_data(self,rss_url,container=None,encoding="utf-8"):
+        """ creates a deferred chain to retrieve the rdf file,
+            parse and extract the metadata and reschedule itself
+        """
+
+        def fail(f):
+            self.info("fail %r", f)
+            self.debug(f.getTraceback())
+            return f
+
+        dfr = getPage(rss_url)
+        dfr.addCallback(parse_xml, encoding=encoding)
+        dfr.addErrback(fail)
+        dfr.addCallback(self.parse_data,container)
+        dfr.addErrback(fail)
+        dfr.addBoth(self.queue_update,rss_url,container)
+        return dfr
+
+    def parse_data(self,xml_data,container):
+        """ extract media info and create BackendItems
+        """
+        pass
+
+    def queue_update(self, error_or_failure,rss_url,container):
+        from twisted.internet import reactor
+        reactor.callLater(self.refresh, self.update_data,rss_url,container)
