@@ -4,6 +4,8 @@
 # Copyright 2008, Frank Scholz <coherence@beebits.net>
 
 import time
+import mimetypes
+mimetypes.init()
 
 try:
     import hashlib
@@ -11,6 +13,10 @@ try:
         m =hashlib.md5()
         m.update(s)
         return m.hexdigest()
+    def sha256(s):
+        m =hashlib.sha256()
+        m.update(s)
+        return m.hexdigest()
 except ImportError:
     import md5 as oldmd5
     def md5(s):
@@ -377,9 +383,13 @@ class Track(BackendItem):
             self.cover = element.find('art').text
         except:
             self.cover = None
+
+        self.mimetype = None
         try:
-            self.mimetype = element.find('mimetype').text
+            self.mimetype = element.find('mime').text
         except:
+            self.mimetype,_ = mimetypes.guess_type(self.url, strict=False)
+        if self.mimetype == None:
             self.mimetype = "audio/mpeg"
         try:
             self.size = int(element.find('size').text)
@@ -473,7 +483,10 @@ class AmpacheStore(BackendStore):
         self.albums = 0
         self.artists = 0
 
-        self.get_token(api_version=350001)
+        self.api_version=int(kwargs.get('api_version',350001))
+        #self.api_version=int(kwargs.get('api_version',340001))
+
+        self.get_token()
 
     def __repr__(self):
         return "Ampache storage"
@@ -502,7 +515,7 @@ class AmpacheStore(BackendStore):
                 return None
         return item
 
-    def got_auth_response( self, response):
+    def got_auth_response( self,response,renegotiate=False):
         self.info( "got_auth_response %r", response)
         try:
             response = utils.parse_xml(response, encoding='utf-8')
@@ -511,11 +524,6 @@ class AmpacheStore(BackendStore):
             raise SyntaxError, 'error parsing ampache answer %r' % msg
         try:
             error = response.find('error').text
-            #if error == 'Error Invalid Handshake, attempt logged':
-            #    """ maybe we've send out the wrong version number,
-            #        let's try it again
-            #    """
-            #    return self.get_token(api_version=350001)
             self.warning('error on token request %r', error)
             raise ValueError, error
         except AttributeError:
@@ -539,49 +547,74 @@ class AmpacheStore(BackendStore):
                 self.info('ampache returned auth token %r', self.token)
                 self.info('Songs: %d, Artists: %d, Albums: %d, Playlists %d, Genres %d, Tags %d' % (self.songs, self.artists,self.albums,self.playlists,self.genres,self.tags))
 
-                self.containers = {}
-                self.containers[ROOT_CONTAINER_ID] = \
-                            Container( ROOT_CONTAINER_ID,-1, self.name, store=self)
+                if renegotiate == False:
+                    self.containers = {}
+                    self.containers[ROOT_CONTAINER_ID] = \
+                                Container( ROOT_CONTAINER_ID,-1, self.name, store=self)
 
-                self.wmc_mapping.update({'4': lambda : self.get_by_id(AUDIO_ALL_CONTAINER_ID),       # all tracks
-                                         '5': lambda : self.get_by_id(AUDIO_GENRE_CONTAINER_ID),     # all genres
-                                         '6': lambda : self.get_by_id(AUDIO_ARTIST_CONTAINER_ID),    # all artists
-                                         '7': lambda : self.get_by_id(AUDIO_ALBUM_CONTAINER_ID),     # all albums
-                                         '13': lambda : self.get_by_id(AUDIO_PLAYLIST_CONTAINER_ID), # all playlists
-                                        })
+                    self.wmc_mapping.update({'4': lambda : self.get_by_id(AUDIO_ALL_CONTAINER_ID),       # all tracks
+                                             '5': lambda : self.get_by_id(AUDIO_GENRE_CONTAINER_ID),     # all genres
+                                             '6': lambda : self.get_by_id(AUDIO_ARTIST_CONTAINER_ID),    # all artists
+                                             '7': lambda : self.get_by_id(AUDIO_ALBUM_CONTAINER_ID),     # all albums
+                                             '13': lambda : self.get_by_id(AUDIO_PLAYLIST_CONTAINER_ID), # all playlists
+                                            })
 
-                louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
+                    louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
             except AttributeError:
                 raise ValueError, 'no authorization token returned'
 
-    def got_auth_error(self, e):
+    def got_auth_error(self,e,renegotiate=False):
         self.warning('error calling ampache %r', e)
-        louie.send('Coherence.UPnP.Backend.init_failed', None, backend=self, msg=e)
+        if renegotiate == False:
+            louie.send('Coherence.UPnP.Backend.init_failed', None, backend=self, msg=e)
 
-    def get_token( self,api_version=None):
+    def get_token(self,renegotiate=False):
         """ ask Ampache for the authorization token """
         timestamp = int(time.time())
-        passphrase = md5('%d%s' % (timestamp, self.key))
+        if self.api_version < 350001:
+            passphrase = md5('%d%s' % (timestamp, self.key))
+        else:
+            passphrase = sha256('%d%s' % (timestamp, sha256(self.key)))
         request = ''.join((self.url, '?action=handshake&auth=%s&timestamp=%d' % (passphrase, timestamp)))
         if self.user != None:
             request = ''.join((request, '&user=%s' % self.user))
-        if api_version != None:
-            request = ''.join((request, '&version=%s' % str(api_version)))
+        if self.api_version != None:
+            request = ''.join((request, '&version=%s' % str(self.api_version)))
         self.info("auth_request %r", request)
         d = utils.getPage(request)
-        d.addCallback(self.got_auth_response)
-        d.addErrback(self.got_auth_error)
+        d.addCallback(self.got_auth_response,renegotiate)
+        d.addErrback(self.got_auth_error,renegotiate)
+        return d
 
     def got_error(self, e):
         self.warning('error calling ampache %r', e)
+        return e
 
-    def got_response(self, response, query_item):
+    def got_response(self, response, query_item, request):
         self.info("got a response for %r", query_item)
         response = utils.parse_xml(response, encoding='utf-8')
         items = []
         try:
-            self.warning('error on token request %r', response.find('error').text)
-            raise ValueError, response.find('error').text
+            error = response.find('error')
+            self.warning('error on token request %r %r' % (error.attrib['code'], error.text))
+            if error.attrib['code'] == '401': # session error, we need to renegotiate our session
+                d = self.get_token(renegotiate=True)
+
+                def resend_request(result, old_request):
+                    # exchange the auth token in the resending request
+                    new_request = old_request.split('&')
+                    for part in new_request:
+                        if part.startswith('auth='):
+                            new_request[new_request.index(part)] = 'auth=%s' % self.token
+                            break
+                    new_request = '&'.join(new_request)
+                    self.info("ampache_query %r", new_request)
+                    return utils.getPage(new_request)
+
+                d.addCallback(resend_request, request)
+                d.addErrBack(self.got_error)
+                return d
+            raise ValueError, error.text
         except AttributeError:
             if query_item in ('song','artist','album','playlist','genre'):
                 q = response.find(query_item)
@@ -630,7 +663,7 @@ class AmpacheStore(BackendStore):
             request = ''.join((request, '&filter=%s' % filter))
         self.info("ampache_query %r", request)
         d = utils.getPage(request)
-        d.addCallback(self.got_response, item)
+        d.addCallback(self.got_response, item, request)
         d.addErrback(self.got_error)
         return d
 
