@@ -13,9 +13,14 @@ import re
 from datetime import datetime
 import urllib
 
+from sets import Set
+
 import mimetypes
 mimetypes.init()
+mimetypes.add_type('audio/x-m4a', '.m4a')
 mimetypes.add_type('video/mp4', '.mp4')
+mimetypes.add_type('video/mpegts', '.ts')
+mimetypes.add_type('video/divx', '.divx')
 
 from urlparse import urlsplit
 
@@ -29,9 +34,14 @@ from coherence.upnp.core.soap_service import errorCode
 
 from coherence.upnp.core import utils
 
-from coherence.extern.inotify import INotify
-from coherence.extern.inotify import IN_CREATE, IN_DELETE, IN_MOVED_FROM, IN_MOVED_TO, IN_ISDIR
-from coherence.extern.inotify import IN_CHANGED
+try:
+    from coherence.extern.inotify import INotify
+    from coherence.extern.inotify import IN_CREATE, IN_DELETE, IN_MOVED_FROM, IN_MOVED_TO, IN_ISDIR
+    from coherence.extern.inotify import IN_CHANGED
+    haz_inotify = True
+except Exception,msg:
+    haz_inotify = False
+    no_inotify_reason = msg
 
 from coherence.extern.xdg import xdg_content
 
@@ -39,10 +49,17 @@ import coherence.extern.louie as louie
 
 from coherence.backend import BackendItem, BackendStore
 
+## Sorting helpers
+NUMS = re.compile('([0-9]+)')
+def _natural_key(s):
+    # strip the spaces
+    s = s.get_name().strip()
+    return [ part.isdigit() and int(part) or part.lower() for part in NUMS.split(s) ]
+
 class FSItem(BackendItem):
     logCategory = 'fs_item'
 
-    def __init__(self, object_id, parent, path, mimetype, urlbase, UPnPClass,update=False):
+    def __init__(self, object_id, parent, path, mimetype, urlbase, UPnPClass,update=False,store=None):
         self.id = object_id
         self.parent = parent
         if parent:
@@ -51,7 +68,7 @@ class FSItem(BackendItem):
             self.location = unicode(path)
         else:
             if mimetype == 'item' and path is None:
-                path = os.path.join(parent.get_path(),unicode(self.id))
+                path = os.path.join(parent.get_realpath(),unicode(self.id))
             #self.location = FilePath(unicode(path))
             self.location = FilePath(path)
         self.mimetype = mimetype
@@ -59,6 +76,7 @@ class FSItem(BackendItem):
             urlbase += '/'
         self.url = urlbase + str(self.id)
 
+        self.store = store
 
         if parent == None:
             parent_id = -1
@@ -70,6 +88,7 @@ class FSItem(BackendItem):
             self.item.childCount = 0
         self.child_count = 0
         self.children = []
+        self.sorted = False
 
 
         if mimetype in ['directory','root']:
@@ -107,6 +126,16 @@ class FSItem(BackendItem):
             except:
                 size = 0
 
+            if self.store.server.coherence.config.get('transcoding', 'no') == 'yes':
+                if self.mimetype in ('application/ogg','audio/ogg',
+                                     'audio/x-wav',
+                                     'audio/x-m4a',
+                                     'application/x-flac'):
+                    new_res = Resource(self.url+'/transcoded.mp3',
+                        'http-get:*:%s:*' % 'audio/mpeg')
+                    new_res.size = None
+                    #self.item.res.append(new_res)
+
             if mimetype != 'item':
                 res = Resource('file://'+ urllib.quote(self.get_path()), 'internal:%s:%s:*' % (host,self.mimetype))
                 res.size = size
@@ -120,6 +149,40 @@ class FSItem(BackendItem):
             res.size = size
             self.item.res.append(res)
 
+            """ if this item is of type audio and we want to add a transcoding rule for it,
+                this is the way to do it:
+
+                create a new Resource object, at least a 'http-get'
+                and maybe an 'internal' one too
+
+                for transcoding to wav this looks like that
+
+                res = Resource(url_for_transcoded audio,
+                        'http-get:*:audio/x-wav:%s'% ';'.join(['DLNA.ORG_PN=JPEG_TN']+simple_dlna_tags))
+                res.size = None
+                self.item.res.append(res)
+            """
+
+            if self.store.server.coherence.config.get('transcoding', 'no') == 'yes':
+                if self.mimetype in ('audio/mpeg',
+                                     'application/ogg','audio/ogg',
+                                     'audio/x-wav',
+                                     'audio/x-m4a',
+                                     'application/x-flac'):
+                    dlna_pn = 'DLNA.ORG_PN=LPCM'
+                    dlna_tags = simple_dlna_tags[:]
+                    #dlna_tags[1] = 'DLNA.ORG_OP=00'
+                    dlna_tags[2] = 'DLNA.ORG_CI=1'
+                    new_res = Resource(self.url+'?transcoded=lpcm',
+                        'http-get:*:%s:%s' % ('audio/L16;rate=44100;channels=2', ';'.join([dlna_pn]+dlna_tags)))
+                    new_res.size = None
+                    #self.item.res.append(new_res)
+
+                    if self.mimetype  != 'audio/mpeg':
+                        new_res = Resource(self.url+'?transcoded=mp3',
+                            'http-get:*:%s:*' % 'audio/mpeg')
+                        new_res.size = None
+                        #self.item.res.append(new_res)
 
             """ if this item is an image and we want to add a thumbnail for it
                 we have to follow these rules:
@@ -130,14 +193,14 @@ class FSItem(BackendItem):
                 for an JPG this looks like that
 
                 res = Resource(url_for_thumbnail,
-                        'http-get:*:image/jpg:%s'% ';'.join(simple_dlna_tags+('DLNA.ORG_PN=JPEG_TN',)))
+                        'http-get:*:image/jpg:%s'% ';'.join(['DLNA.ORG_PN=JPEG_TN']+simple_dlna_tags))
                 res.size = size_of_thumbnail
                 self.item.res.append(res)
 
                 and for a PNG the Resource creation is like that
 
                 res = Resource(url_for_thumbnail,
-                        'http-get:*:image/png:%s'% ';'.join(simple_dlna_tags+('DLNA.ORG_PN=PNG_TN',)))
+                        'http-get:*:image/png:%s'% ';'.join(simple_dlna_tags+['DLNA.ORG_PN=PNG_TN']))
 
                 if not hasattr(self.item, 'attachments'):
                     self.item.attachments = {}
@@ -155,9 +218,12 @@ class FSItem(BackendItem):
                         else:
                             dlna_pn = 'DLNA.ORG_PN=PNG_TN'
 
+                        dlna_tags = simple_dlna_tags[:]
+                        dlna_tags[3] = 'DLNA.ORG_FLAGS=00f00000000000000000000000000000'
+
                         hash_from_path = str(id(thumbnail))
                         new_res = Resource(self.url+'?attachment='+hash_from_path,
-                            'http-get:*:%s:%s' % (mimetype, ';'.join(simple_dlna_tags+(dlna_pn,))))
+                            'http-get:*:%s:%s' % (mimetype, ';'.join([dlna_pn]+dlna_tags)))
                         new_res.size = os.path.getsize(thumbnail)
                         self.item.res.append(new_res)
                         if not hasattr(self.item, 'attachments'):
@@ -248,6 +314,7 @@ class FSItem(BackendItem):
             self.item.childCount += 1
         if update == True:
             self.update_id += 1
+        self.sorted = False
 
     def remove_child(self, child):
         #print "remove_from %d (%s) child %d (%s)" % (self.id, self.get_name(), child.id, child.get_name())
@@ -257,8 +324,12 @@ class FSItem(BackendItem):
                 self.item.childCount -= 1
             self.children.remove(child)
             self.update_id += 1
+        self.sorted = False
 
     def get_children(self,start=0,request_count=0):
+        if self.sorted == False:
+            self.children.sort(key=_natural_key)
+            self.sorted = True
         if request_count == 0:
             return self.children[start:]
         else:
@@ -282,6 +353,12 @@ class FSItem(BackendItem):
         else:
             self.location
 
+    def get_realpath(self):
+        if isinstance( self.location,FilePath):
+            return self.location.path
+        else:
+            self.location
+
     def set_path(self,path=None,extension=None):
         if path is None:
             path = self.get_path()
@@ -294,7 +371,7 @@ class FSItem(BackendItem):
             self.location = path
 
     def get_name(self):
-        if isinstance( self.location,FilePath):
+        if isinstance(self.location,FilePath):
             name = self.location.basename().decode("utf-8", "replace")
         else:
             name = self.location.decode("utf-8", "replace")
@@ -340,45 +417,70 @@ class FSStore(BackendStore):
                 self.content = l
         else:
             self.content = xdg_content()
+            self.content = [x[0] for x in self.content]
         if self.content == None:
             self.content = 'tests/content'
         if not isinstance( self.content, list):
             self.content = [self.content]
+        self.content = Set([os.path.abspath(x) for x in self.content])
         self.urlbase = kwargs.get('urlbase','')
         ignore_patterns = kwargs.get('ignore_patterns',[])
 
-        if self.urlbase[len(self.urlbase)-1] != '/':
+        if not self.urlbase.endswith('/'):
             self.urlbase += '/'
         self.server = server
         self.store = {}
 
-        try:
-            self.inotify = INotify()
-        except:
-            self.inotify = None
+        self.inotify = None
+
+        if haz_inotify == True:
+            try:
+                self.inotify = INotify()
+            except Exception,msg:
+                self.info("%s" %msg)
+        else:
+            self.info("%s" %no_inotify_reason)
 
         if kwargs.get('enable_destroy','no') == 'yes':
             self.upnp_DestroyObject = self.hidden_upnp_DestroyObject
 
+        self.import_folder = kwargs.get('import_folder',None)
+        if self.import_folder != None:
+            self.import_folder = os.path.abspath(self.import_folder)
+            if not os.path.isdir(self.import_folder):
+                self.import_folder = None
+
         self.ignore_file_pattern = re.compile('|'.join(['^\..*'] + list(ignore_patterns)))
         parent = None
         self.update_id = 0
-        if len(self.content)>1:
+        if(len(self.content)>1 or
+           utils.means_true(kwargs.get('create_root',False)) or
+           self.import_folder != None):
             UPnPClass = classChooser('root')
-            id = self.getnextID()
-            parent = self.store[id] = FSItem( id, parent, 'media', 'root', self.urlbase, UPnPClass, update=True)
+            id = str(self.getnextID())
+            parent = self.store[id] = FSItem( id, parent, 'media', 'root', self.urlbase, UPnPClass, update=True,store=self)
+
+        if self.import_folder != None:
+            id = str(self.getnextID())
+            self.store[id] = FSItem( id, parent, self.import_folder, 'directory', self.urlbase, UPnPClass, update=True,store=self)
+            self.import_folder_id = id
 
         for path in self.content:
             if isinstance(path,(list,tuple)):
                 path = path[0]
             if self.ignore_file_pattern.match(path):
                 continue
-            self.walk(path, parent, self.ignore_file_pattern)
+            try:
+                self.walk(path, parent, self.ignore_file_pattern)
+            except Exception,msg:
+                self.warning('on walk of %r: %r' % (path,msg))
+                import traceback
+                self.debug(traceback.format_exc())
 
-        self.wmc_mapping.update({'14': 0,
-                                 '15': 0,
-                                 '16': 0,
-                                 '17': 0
+        self.wmc_mapping.update({'14': '0',
+                                 '15': '0',
+                                 '16': '0',
+                                 '17': '0'
                                 })
 
         louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
@@ -394,36 +496,82 @@ class FSStore(BackendStore):
         return len(self.store)
 
     def get_by_id(self,id):
+        #print "get_by_id", id, type(id)
         # we have referenced ids here when we are in WMC mapping mode
         if isinstance(id, basestring):
             id = id.split('@',1)
             id = id[0]
-        try:
-            id = int(id)
-        except ValueError:
-            id = 1000
+        #try:
+        #    id = int(id)
+        #except ValueError:
+        #    id = 1000
 
-        if id == 0:
-            id = 1000
+        if id == '0':
+            id = '1000'
+        #print "get_by_id 2", id
         try:
-            return self.store[id]
+            r = self.store[id]
         except:
-            return None
+            r = None
+        #print "get_by_id 3", r
+        return r
 
-    def get_id_by_name(self, parent, name):
+    def get_id_by_name(self, parent='0', name=''):
+        self.info('get_id_by_name %r (%r) %r' % (parent, type(parent), name))
         try:
-            parent = self.store[int(parent)]
+            parent = self.store[parent]
+            self.debug("%r %d" % (parent,len(parent.children)))
             for child in parent.children:
-                if not isinstance(name, unicode):
-                    name = name.decode("utf8")
-                if name == child.get_name():
+                #if not isinstance(name, unicode):
+                #    name = name.decode("utf8")
+                self.debug("%r %r %r" % (child.get_name(),child.get_realpath(), name == child.get_realpath()))
+                if name == child.get_realpath():
                     return child.id
         except:
-            pass
+            import traceback
+            self.info(traceback.format_exc())
+        self.debug('get_id_by_name not found')
 
         return None
 
+    def get_url_by_name(self,parent='0',name=''):
+        self.info('get_url_by_name %r %r' % (parent, name))
+        id = self.get_id_by_name(parent,name)
+        #print 'get_url_by_name', id
+        if id == None:
+            return ''
+        return self.store[id].url
+
+
+    def update_config(self,**kwargs):
+        print "update_config", kwargs
+        if 'content' in kwargs:
+            new_content = kwargs['content']
+            new_content = Set([os.path.abspath(x) for x in new_content.split(',')])
+            new_folders = new_content.difference(self.content)
+            obsolete_folders = self.content.difference(new_content)
+            print new_folders, obsolete_folders
+            for folder in obsolete_folders:
+                self.remove_content_folder(folder)
+            for folder in new_folders:
+                self.add_content_folder(folder)
+            self.content = new_content
+
+    def add_content_folder(self,path):
+        path = os.path.abspath(path)
+        if path not in self.content:
+            self.content.add(path)
+            self.walk(path, self.store['1000'], self.ignore_file_pattern)
+
+    def remove_content_folder(self,path):
+        path = os.path.abspath(path)
+        if path in self.content:
+            id = self.get_id_by_name('1000', path)
+            self.remove(id)
+            self.content.remove(path)
+
     def walk(self, path, parent=None, ignore_file_pattern=''):
+        self.debug("walk %r" % path)
         containers = []
         parent = self.append(path,parent)
         if parent != None:
@@ -431,6 +579,7 @@ class FSStore(BackendStore):
         while len(containers)>0:
             container = containers.pop()
             try:
+                self.debug('adding %r' % container.location)
                 for child in container.location.children():
                     if ignore_file_pattern.match(child.basename()) != None:
                         continue
@@ -441,17 +590,22 @@ class FSStore(BackendStore):
                 self.warning("UnicodeDecodeError - there is something wrong with a file located in %r", container.get_path())
 
     def create(self, mimetype, path, parent):
-        #print "create", mimetype, path, type(path), parent
+        self.debug("create ", mimetype, path, type(path), parent)
         UPnPClass = classChooser(mimetype)
         if UPnPClass == None:
             return None
 
         id = self.getnextID()
+        if mimetype in ('root','directory'):
+            id = str(id)
+        else:
+            _,ext =  os.path.splitext(path)
+            id = str(id) + ext.lower()
         update = False
         if hasattr(self, 'update_id'):
             update = True
 
-        self.store[id] = FSItem( id, parent, path, mimetype, self.urlbase, UPnPClass, update=True)
+        self.store[id] = FSItem( id, parent, path, mimetype, self.urlbase, UPnPClass, update=True,store=self)
         if hasattr(self, 'update_id'):
             self.update_id += 1
             #print self.update_id
@@ -467,7 +621,10 @@ class FSStore(BackendStore):
         return id
 
     def append(self,path,parent):
-        #print "append", path, type(path), parent
+        self.debug("append ", path, type(path), parent)
+        if os.path.exists(path) == False:
+            self.warning("path %r not available - ignored", path)
+            return None
         try:
             mimetype,_ = mimetypes.guess_type(path, strict=False)
             if mimetype == None:
@@ -490,12 +647,12 @@ class FSStore(BackendStore):
         return None
 
     def remove(self, id):
-        #print 'FSSTore remove id', id
+        print 'FSSTore remove id', id
         try:
-            item = self.store[int(id)]
+            item = self.store[id]
             parent = item.get_parent()
             item.remove()
-            del self.store[int(id)]
+            del self.store[id]
             if hasattr(self, 'update_id'):
                 self.update_id += 1
                 if self.server:
@@ -510,8 +667,8 @@ class FSStore(BackendStore):
 
 
     def notify(self, iwp, filename, mask, parameter=None):
-        #print "Event %s on %s %s - id %d" % (
-        #    ', '.join(self.inotify.flag_to_human(mask)), iwp.path, filename, parameter)
+        self.info("Event %s on %s %s - parameter %r" % (
+                    ', '.join(self.inotify.flag_to_human(mask)), iwp.path, filename, parameter))
 
         path = iwp.path
         if filename:
@@ -523,15 +680,15 @@ class FSStore(BackendStore):
             pass
 
         if(mask & IN_DELETE or mask & IN_MOVED_FROM):
-            #print '%s was deleted, parent %d (%s)' % (path, parameter, iwp.path)
-            id = self.get_id_by_name(parameter,filename)
+            self.info('%s was deleted, parent %r (%s)' % (path, parameter, iwp.path))
+            id = self.get_id_by_name(parameter,os.path.join(iwp.path,filename))
             self.remove(id)
         if(mask & IN_CREATE or mask & IN_MOVED_TO):
-            #if mask & IN_ISDIR:
-            #    print 'directory %s was created, parent %d (%s)' % (path, parameter, iwp.path)
-            #else:
-            #    print 'file %s was created, parent %d (%s)' % (path, parameter, iwp.path)
-            if self.get_id_by_name(parameter,filename) is None:
+            if mask & IN_ISDIR:
+                self.info('directory %s was created, parent %r (%s)' % (path, parameter, iwp.path))
+            else:
+                self.info('file %s was created, parent %r (%s)' % (path, parameter, iwp.path))
+            if self.get_id_by_name(parameter,os.path.join(iwp.path,filename)) is None:
                 if os.path.isdir(path):
                     self.walk(path, self.get_by_id(parameter), self.ignore_file_pattern)
                 else:
@@ -542,11 +699,35 @@ class FSStore(BackendStore):
         self.next_id += 1
         return ret
 
+    def backend_import(self,item,data):
+        try:
+            f = open(item.get_path(), 'w+b')
+            if hasattr(data,'read'):
+                data = data.read()
+            f.write(data)
+            f.close()
+            item.rebuild(self.urlbase)
+            return 200
+        except IOError:
+            self.warning("import of file %s failed" % item.get_path())
+        except Exception,msg:
+            import traceback
+            self.warning(traceback.format_exc())
+        return 500
+
     def upnp_init(self):
         self.current_connection_id = None
         if self.server:
             self.server.connection_manager_server.set_variable(0, 'SourceProtocolInfo',
-                        ['internal:%s:audio/mpeg:*' % self.server.coherence.hostname,
+                        [#'http-get:*:audio/mpeg:DLNA.ORG_PN=MP3;DLNA.ORG_OP=11;DLNA.ORG_FLAGS=01700000000000000000000000000000',
+                         #'http-get:*:audio/x-ms-wma:DLNA.ORG_PN=WMABASE;DLNA.ORG_OP=11;DLNA.ORG_FLAGS=01700000000000000000000000000000',
+                         #'http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_TN;DLNA.ORG_OP=01;DLNA.ORG_FLAGS=00f00000000000000000000000000000',
+                         #'http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_SM;DLNA.ORG_OP=01;DLNA.ORG_FLAGS=00f00000000000000000000000000000',
+                         #'http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_MED;DLNA.ORG_OP=01;DLNA.ORG_FLAGS=00f00000000000000000000000000000',
+                         #'http-get:*:image/jpeg:DLNA.ORG_PN=JPEG_LRG;DLNA.ORG_OP=01;DLNA.ORG_FLAGS=00f00000000000000000000000000000',
+                         #'http-get:*:video/mpeg:DLNA.ORG_PN=MPEG_PS_PAL;DLNA.ORG_OP=01;DLNA.ORG_FLAGS=01700000000000000000000000000000',
+                         #'http-get:*:video/x-ms-wmv:DLNA.ORG_PN=WMVMED_BASE;DLNA.ORG_OP=01;DLNA.ORG_FLAGS=01700000000000000000000000000000',
+                         'internal:%s:audio/mpeg:*' % self.server.coherence.hostname,
                          'http-get:*:audio/mpeg:*',
                          'internal:%s:video/mp4:*' % self.server.coherence.hostname,
                          'http-get:*:video/mp4:*',
@@ -557,9 +738,14 @@ class FSStore(BackendStore):
                          'internal:%s:video/avi:*' % self.server.coherence.hostname,
                          'http-get:*:video/avi:*',
                          'internal:%s:video/quicktime:*' % self.server.coherence.hostname,
-                         'http-get:*:video/quicktime:*'],
+                         'http-get:*:video/quicktime:*',
+                         'internal:%s:image/gif:*' % self.server.coherence.hostname,
+                         'http-get:*:image/gif:*',
+                         'internal:%s:image/jpeg:*' % self.server.coherence.hostname,
+                         'http-get:*:image/jpeg:*'],
                         default=True)
             self.server.content_directory_server.set_variable(0, 'SystemUpdateID', self.update_id)
+            #self.server.content_directory_server.set_variable(0, 'SortCapabilities', '*')
 
 
     def upnp_ImportResource(self, *args, **kwargs):
@@ -614,9 +800,12 @@ class FSStore(BackendStore):
         return {'TransferID': transfer_id}
 
     def upnp_CreateObject(self, *args, **kwargs):
+        #print "CreateObject", kwargs
         if kwargs['ContainerID'] == 'DLNA.ORG_AnyContainer':
-            """ for the moment """
-            return failure.Failure(errorCode(712))
+            if self.import_folder != None:
+                ContainerID = self.import_folder_id
+            else:
+                return failure.Failure(errorCode(712))
         else:
             ContainerID = kwargs['ContainerID']
         Elements = kwargs['Elements']
@@ -635,8 +824,10 @@ class FSStore(BackendStore):
             return failure.Failure(errorCode(712))
 
         item = elt.getItems()[0]
+        if item.parentID == 'DLNA.ORG_AnyContainer':
+            item.parentID = ContainerID
         if(item.id != '' or
-           int(item.parentID) != ContainerID or
+           item.parentID != ContainerID or
            item.restricted == True or
            item.title == ''):
             return failure.Failure(errorCode(712))
@@ -666,9 +857,11 @@ class FSStore(BackendStore):
             didl.addItem(new_item.item)
             return {'ObjectID': id, 'Result': didl.toString()}
 
-        if item.upnp_class.startswith('object.item.'):
-            #path = os.path.join(parent_item.get_path(),item.title)
-            id = self.create('item',None,parent_item)
+        if item.upnp_class.startswith('object.item'):
+            _,_,content_format,_ = item.res[0].protocolInfo.split(':')
+            extension = mimetypes.guess_extension(content_format, strict=False)
+            path = os.path.join(parent_item.get_realpath(),item.title+extension)
+            id = self.create('item',path,parent_item)
 
             new_item = self.get_by_id(id)
             for res in new_item.item.res:
