@@ -61,6 +61,13 @@ from coherence.extern.covers_by_amazon import CoverGetter
 
 from coherence.backend import BackendItem, BackendStore
 
+KNOWN_AUDIO_TYPES = {'.mp3':'audio/mpeg',
+                     '.ogg':'application/ogg',
+                     '.mpc':'audio/x-musepack',
+                     '.flac':'audio/x-wavpack',
+                     '.wv':'audio/x-wavpack',
+                     '.m4a':'audio/mp4',}
+
 
 def _dict_from_tags(tag):
     tags = {}
@@ -190,15 +197,22 @@ class Artist(item.Item,BackendItem):
 
     get_path = None
 
+    def get_artist_all_tracks(self,start=0,request_count=0):
+        children = [x[1] for x in list(self.store.query((Album,Track),
+                            attributes.AND(Album.artist == self,
+                                           Track.album == Album.storeID),
+                            sort=(Album.title.ascending,Track.track_nr.ascending)
+                            ))]
+        if request_count == 0:
+            return children[start:]
+        else:
+            return children[start:request_count]
+
     def get_children(self,start=0,request_count=0):
         all_id = 'artist_all_tracks_%d' % (self.storeID+1000)
         self.store.containers[all_id] = \
                 Container( all_id, self.storeID+1000, 'All tracks of %s' % self.name,
-                          children_callback=lambda :[x[1] for x in list(self.store.query((Album,Track),
-                            attributes.AND(Album.artist == self,
-                                           Track.album == Album.storeID),
-                            sort=(Track.title.ascending)
-                            ))])
+                          children_callback=self.get_artist_all_tracks)
 
         children = [self.store.containers[all_id]] + list(self.store.query(Album, Album.artist == self,sort=Album.title.ascending))
         if request_count == 0:
@@ -315,9 +329,11 @@ class Track(item.Item,BackendItem):
             host = host_port
 
         _,ext =  os.path.splitext(self.location)
-        if ext == '.ogg':
-            mimetype = 'application/ogg'
-        else:
+        ext = ext.lower()
+
+        try:
+            mimetype = KNOWN_AUDIO_TYPES[ext]
+        except KeyError:
             mimetype = 'audio/mpeg'
 
         statinfo = os.stat(self.location)
@@ -329,7 +345,7 @@ class Track(item.Item,BackendItem):
             res.size = 0
         item.res.append(res)
 
-        url = self.store.urlbase + str(self.storeID+1000)
+        url = self.store.urlbase + str(self.storeID+1000) + ext
 
         res = DIDLLite.Resource(url, 'http-get:*:%s:*' % mimetype)
         try:
@@ -338,16 +354,37 @@ class Track(item.Item,BackendItem):
             res.size = 0
         item.res.append(res)
 
+        #if self.store.server.coherence.config.get('transcoding', 'no') == 'yes':
+        #    if mimetype in ('audio/mpeg',
+        #                    'application/ogg','audio/ogg',
+        #                    'audio/x-m4a',
+        #                    'application/x-flac'):
+        #        dlna_pn = 'DLNA.ORG_PN=LPCM'
+        #        dlna_tags = DIDLLite.simple_dlna_tags[:]
+        #        dlna_tags[1] = 'DLNA.ORG_CI=1'
+        #        #dlna_tags[2] = 'DLNA.ORG_OP=00'
+        #        new_res = DIDLLite.Resource(url+'?transcoded=lpcm',
+        #            'http-get:*:%s:%s' % ('audio/L16;rate=44100;channels=2', ';'.join([dlna_pn]+dlna_tags)))
+        #        new_res.size = None
+        #        item.res.append(new_res)
+        #
+        #        if mimetype != 'audio/mpeg':
+        #            new_res = DIDLLite.Resource(url+'?transcoded=mp3',
+        #                'http-get:*:%s:*' % 'audio/mpeg')
+        #            new_res.size = None
+        #            item.res.append(new_res)
+
         try:
             # FIXME: getmtime is deprecated in Twisted 2.6
             item.date = datetime.fromtimestamp(statinfo.st_mtime)
         except:
             item.date = None
 
+
         return item
 
     def get_path(self):
-        return self.location
+        return self.location.encode('utf-8')
 
     def get_id(self):
         return self.storeID + 1000
@@ -429,7 +466,7 @@ class MediaStore(BackendStore):
                     self.walk(os.path.join(path,filename))
                 else:
                     _,ext =  os.path.splitext(filename)
-                    if ext.lower()[1:] in ['mp3','ogg']:
+                    if ext.lower() in KNOWN_AUDIO_TYPES:
                         self.filelist.append(os.path.join(path,filename))
 
     def get_music_files(self, musiclocation):
@@ -569,7 +606,7 @@ class MediaStore(BackendStore):
         self.info("get_by_id %s" % id)
         if isinstance(id, basestring):
             id = id.split('@',1)
-            id = id[0]
+            id = id[0].split('.')[0]
         if isinstance(id, basestring) and id.startswith('artist_all_tracks_'):
             try:
                 return self.containers[id]
