@@ -7,11 +7,12 @@ import string
 import socket
 import os, sys
 import traceback
+import copy
 
 from twisted.python import filepath, util
 from twisted.internet import task, address, defer
 from twisted.internet import reactor
-from twisted.web import resource
+from twisted.web import resource,static
 
 import coherence.extern.louie as louie
 
@@ -43,6 +44,9 @@ class SimpleRoot(resource.Resource, log.Loggable):
 
     def getChild(self, name, request):
         self.debug('SimpleRoot getChild %s, %s' % (name, request))
+        if name == 'oob':
+            """ we have an out-of-band request """
+            return static.File(self.coherence.dbus.pinboard[request.args['key'][0]])
         try:
             return self.coherence.children[name]
         except:
@@ -161,19 +165,25 @@ class Plugins(log.Loggable):
                     try:
                         #print entrypoint, type(entrypoint)
                         self._plugins[entrypoint.name] = entrypoint.load(require=False)
-                    except ImportError, msg:
+                    except (ImportError, pkg_resources.ResolutionError), msg:
                         self.warning("Can't load plugin %s (%s), maybe missing dependencies..." % (entrypoint.name,msg))
                         self.info(traceback.format_exc())
         except ImportError:
-            self.info("plugin reception activated, no pkg_resources")
-            from coherence.extern.simple_plugin import Reception
-            reception = Reception(os.path.join(os.path.dirname(__file__),'backends'), log=self.warning)
-            self.info(reception.guestlist())
-            for cls in reception.guestlist():
-                self._plugins[cls.__name__.split('.')[-1]] = cls
+            self.info("no pkg_resources, fallback to simple plugin handling")
+
         except Exception, msg:
             self.warning(msg)
 
+        if len(self._plugins) == 0:
+            self._collect_from_module()
+
+    def _collect_from_module(self):
+        from coherence.extern.simple_plugin import Reception
+        reception = Reception(os.path.join(os.path.dirname(__file__),'backends'), log=self.warning)
+        self.info(reception.guestlist())
+        for cls in reception.guestlist():
+            self._plugins[cls.__name__.split('.')[-1]] = cls
+
 
 class Coherence(log.Loggable):
     logCategory = 'coherence'
@@ -182,10 +192,12 @@ class Coherence(log.Loggable):
     def __new__(cls, *args, **kwargs):
         obj = getattr(cls, '_instance_', None)
         if obj is not None:
+            cls._incarnations_ += 1
             return obj
         else:
-            obj = super(Coherence, cls).__new__(cls, *args, **kwargs)
+            obj = super(Coherence, cls).__new__(cls)
             cls._instance_ = obj
+            cls._incarnations_ = 1
             obj.setup(*args, **kwargs)
             obj.cls = cls
             return obj
@@ -219,16 +231,24 @@ class Coherence(log.Loggable):
         """
 
         try:
-            logmode = config['logging'].get('level','warning')
-        except KeyError:
+            logmode = config.get('logging').get('level','warning')
+        except (KeyError,AttributeError):
             logmode = config.get('logmode', 'warning')
         _debug = []
 
         try:
-            for subsystem in config['logging'].get('subsystemlist',[]):
+            subsystems = config.get('logging')['subsystem']
+            if isinstance(subsystems,dict):
+                subsystems = [subsystems]
+            for subsystem in subsystems:
+                try:
+                    if subsystem['active'] == 'no':
+                        continue
+                except (KeyError,TypeError):
+                    pass
                 self.info( "setting log-level for subsystem %s to %s" % (subsystem['name'],subsystem['level']))
                 _debug.append('%s:%d' % (subsystem['name'].lower(), log.human2level(subsystem['level'])))
-        except KeyError:
+        except (KeyError,TypeError):
             subsystem_log = config.get('subsystem_log',{})
             for subsystem,level in subsystem_log.items():
                 #self.info( "setting log-level for subsystem %s to %s" % (subsystem,level))
@@ -237,7 +257,11 @@ class Coherence(log.Loggable):
             _debug = ','.join(_debug)
         else:
             _debug = '*:%d' % log.human2level(logmode)
-        log.init(config.get('logfile', None), _debug)
+        try:
+            logfile = config.get('logging').get('logfile',None)
+        except (KeyError,AttributeError,TypeError):
+            logfile = config.get('logfile', None)
+        log.init(logfile, _debug)
 
         self.louieplugin = louie.TwistedDispatchPlugin()
         try:
@@ -250,9 +274,11 @@ class Coherence(log.Loggable):
 
         self.warning("Coherence UPnP framework version %s starting..." % __version__)
 
-        unittest = config.get('unittest', False)
-        if unittest != False:
-            unittest == True
+        unittest = config.get('unittest', 'no')
+        if unittest == 'no':
+            unittest = False
+        else:
+            unittest = True
 
         self.ssdp_server = SSDPServer(test=unittest)
         louie.connect( self.create_device, 'Coherence.UPnP.SSDP.new_device', louie.Any)
@@ -265,10 +291,10 @@ class Coherence(log.Loggable):
 
         self.msearch = MSearch(self.ssdp_server,test=unittest)
 
-        reactor.addSystemEventTrigger( 'before', 'shutdown', self.shutdown)
+        reactor.addSystemEventTrigger( 'before', 'shutdown', self.shutdown, force=True)
 
         if network_if:
-            self.hostname = get_ip_address(network_if)
+            self.hostname = get_ip_address('%s' % network_if)
         else:
             try:
                 self.hostname = socket.gethostbyname(socket.gethostname())
@@ -292,7 +318,12 @@ class Coherence(log.Loggable):
 
         self.available_plugins = None
 
-        plugins = config.get('pluginlist')
+        try:
+            plugins = config['plugin']
+            if isinstance(plugins,dict):
+                plugins=[plugins]
+        except:
+            plugins = None
         if plugins is None:
             plugins = config.get('plugins',None)
 
@@ -310,10 +341,20 @@ class Coherence(log.Loggable):
                         self.info(traceback.format_exc())
             else:
                 for plugin in plugins:
+                    try:
+                        if plugin['active'] == 'no':
+                            continue
+                    except (KeyError,TypeError):
+                        pass
                     try:
                         backend = plugin['backend']
-                        del plugin['backend']
-                        self.add_plugin(backend, **plugin)
+                        arguments = copy.copy(plugin)
+                        del arguments['backend']
+                        backend = self.add_plugin(backend, **arguments)
+                        if self.writeable_config() == True:
+                            if 'uuid' not in plugin:
+                                plugin['uuid'] = str(backend.uuid)[5:]
+                                self.config.save()
                     except Exception, msg:
                         self.warning("Can't enable plugin, %s: %s!" % (plugin, msg))
                         self.info(traceback.format_exc())
@@ -371,7 +412,7 @@ class Coherence(log.Loggable):
                 return ""
 
         try:
-            del self.active_backends[plugin.uuid]
+            del self.active_backends[str(plugin.uuid)]
             self.info("removing plugin %r", plugin)
             plugin.unregister()
             return plugin.uuid
@@ -379,12 +420,52 @@ class Coherence(log.Loggable):
             self.warning("no backend with the uuid %r found" % plugin.uuid)
             return ""
 
+    def writeable_config(self):
+        """ do we have a new-style config file """
+        from coherence.extern.simple_config import ConfigItem
+        if isinstance(self.config,ConfigItem):
+            return True
+        return False
+
+    def store_plugin_config(self,uuid,items):
+        """ find the backend with uuid
+            and store in its the config
+            the key and value pair(s)
+        """
+        plugins = self.config.get('plugin')
+        if plugins is None:
+            self.warning("storing a plugin config option is only possible with the new config file format")
+            return
+        if isinstance(plugins,dict):
+            plugins = [plugins]
+        uuid = str(uuid)
+        if uuid.startswith('uuid:'):
+            uuid = uuid[5:]
+        if isinstance(items,tuple):
+            new = {}
+            new[items[0]] = items[1]
+        for plugin in plugins:
+            try:
+                if plugin['uuid'] == uuid:
+                    for k,v in items.items():
+                        plugin[k] = v
+                    self.config.save()
+            except:
+                pass
+        else:
+            self.info("storing plugin config option for %s failed, plugin not found" % uuid)
+
     def receiver( self, signal, *args, **kwargs):
         #print "Coherence receiver called with", signal
         #print kwargs
         pass
 
-    def shutdown( self):
+    def shutdown( self,force=False):
+        if force == True:
+            self._incarnations_ = 1
+        if self._incarnations_ > 1:
+            self._incarnations_ -= 1
+            return
         if self.louieplugin != None:
             louie.remove_plugin(self.louieplugin)
             self.louieplugin = None
@@ -505,6 +586,7 @@ class Coherence(log.Loggable):
             device.remove()
             if infos['ST'] == 'upnp:rootdevice':
                 louie.send('Coherence.UPnP.Device.removed', None, usn=infos['USN'])
+                louie.send('Coherence.UPnP.RootDevice.removed', None, usn=infos['USN'])
                 self.callback("removed_device", infos['ST'], infos['USN'])
 
 
@@ -512,4 +594,18 @@ class Coherence(log.Loggable):
         self.children[name] = sub
 
     def remove_web_resource(self, name):
-        del self.children[name]
+        try:
+            del self.children[name]
+        except KeyError:
+            """ probably the backend init failed """
+            pass
+
+    def connect(self,receiver,signal=louie.signal.All,sender=louie.sender.Any, weak=True):
+        """ wrapper method around louie.connect
+        """
+        louie.connect(receiver,signal=signal,sender=sender,weak=weak)
+
+    def disconnect(self,receiver,signal=louie.signal.All,sender=louie.sender.Any, weak=True):
+        """ wrapper method around louie.disconnect
+        """
+        louie.disconnect(receiver,signal=signal,sender=sender,weak=weak)
