@@ -26,6 +26,21 @@ from coherence.upnp.core import dlna
 
 from coherence import log
 
+
+def is_audio(mimetype):
+    """ checks for type audio,
+        expects a mimetype or an UPnP
+        protocolInfo
+    """
+    test = mimetype.split(':')
+    if len(test) == 4:
+        mimetype = test[2]
+    if mimetype == 'application/ogg':
+        return True
+    if mimetype.startswith('audio/'):
+        return True
+    return False
+
 class Resources(list):
 
     """ a list of resources, always sorted after an append """
@@ -69,6 +84,8 @@ class Resources(list):
         if not isinstance(local_protocol_infos, list):
             local_protocol_infos = [local_protocol_infos]
         for res in self:
+            if res.importUri != None:
+                continue
             #print "res", res.protocolInfo, res.data
             remote_protocol,remote_network,remote_content_format,_ = res.protocolInfo.split(':')
             #print "remote", remote_protocol,remote_network,remote_content_format
@@ -114,12 +131,44 @@ def classChooser(mimetype, sub=None):
             if sub == 'music':       # FIXME: this is stupid
                 return MusicTrack
             return AudioItem
+        if mimetype == 'application/x-flac':
+            if sub == 'music':       # FIXME: this is stupid
+                return MusicTrack
+            return AudioItem
     return None
 
-simple_dlna_tags = ('DLNA.ORG.PS=1',       # play speed parameter
+simple_dlna_tags = ['DLNA.ORG_OP=01',      # operations parameter
+                    'DLNA.ORG_PS=1',       # play speed parameter
                     'DLNA.ORG_CI=0',       # transcoded parameter
-                    'DLNA.ORG_OP=01',      # operations parameter
-                    'DLNA.ORG_FLAGS=01700000000000000000000000000000')
+                    'DLNA.ORG_FLAGS=01100000000000000000000000000000']
+
+def build_dlna_additional_info(content_format):
+    additional_info = '*'
+    if content_format == 'audio/mpeg':
+        additional_info = ';'.join(['DLNA.ORG_PN=MP3']+simple_dlna_tags)
+    if content_format == 'audio/ms-wma':
+        additional_info = ';'.join(['DLNA.ORG_PN=WMABASE']+simple_dlna_tags)
+    if content_format == 'image/jpeg':
+        dlna_tags = simple_dlna_tags[:]
+        dlna_tags[3] = 'DLNA.ORG_FLAGS=00900000000000000000000000000000'
+        additional_info = ';'.join(['DLNA.ORG_PN=JPEG_LRG']+dlna_tags)
+    if content_format == 'image/png':
+        dlna_tags = simple_dlna_tags[:]
+        dlna_tags[3] = 'DLNA.ORG_FLAGS=00900000000000000000000000000000'
+        additional_info = ';'.join(['DLNA.ORG_PN=PNG_LRG']+dlna_tags)
+    if content_format == 'video/mpeg':
+        additional_info = ';'.join(['DLNA.ORG_PN=MPEG_PS_PAL']+simple_dlna_tags)
+    if content_format == 'video/mpegts':
+        additional_info = ';'.join(['DLNA.ORG_PN=MPEG_TS_PAL']+simple_dlna_tags)
+        content_format = 'video/mpeg'
+    if content_format in ['video/mp4','video/x-m4a']:
+        additional_info = ';'.join(['DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC']+simple_dlna_tags)
+    if content_format == 'video/x-msvideo':
+        #additional_info = ';'.join(['DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC']+simple_dlna_tags)
+        additional_info = '*'
+    if content_format == 'video/x-ms-wmv':
+        additional_info = ';'.join(['DLNA.ORG_PN=WMV_BASE']+simple_dlna_tags)
+    return additional_info
 
 
 class Resource:
@@ -132,41 +181,51 @@ class Resource:
         self.size = None
         self.duration = None
 
+        self.resolution = None
+
         self.importUri = None
 
         if self.protocolInfo is not None:
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
             if additional_info == '*':
-                if content_format == 'audio/mpeg':
-                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=MP3',))
-                if content_format == 'image/jpeg':
-                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=JPEG_LRG',))
-                if content_format == 'image/png':
-                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=PNG_LRG',))
-                if content_format == 'video/mpeg':
-                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=MPEG_PS_PAL',))
-                if content_format == 'video/mp4':
-                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC',))
-                if content_format == 'video/x-msvideo':
-                    additional_info = ';'.join(simple_dlna_tags+('DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC',))
+                self.protocolInfo = ':'.join((protocol,network,content_format,build_dlna_additional_info(content_format)))
 
-                self.protocolInfo = ':'.join((protocol,network,content_format,additional_info))
+    def get_additional_info(self,upnp_client=''):
+        protocol,network,content_format,additional_info = self.protocolInfo.split(':')
+        if upnp_client  in ('XBox','Philips-TV',):
+            """ we don't need the DLNA tags there,
+                and maybe they irritates these poor things anyway
+            """
+            additional_info = '*'
+        elif upnp_client  in ('PLAYSTATION3',):
+            if content_format.startswith('video/'):
+                additional_info = '*'
+
+        a_list = additional_info.split(';')
+        for part in a_list:
+            if part == 'DLNA.ORG_PS=1':
+                a_list.remove(part)
+                break
+        additional_info = ';'.join(a_list)
+        return additional_info
 
     def toElement(self,**kwargs):
-
         root = ET.Element('res')
-        if kwargs.get('upnp_client','') in ('XBox', 'PLAYSTATION3'):
+        if kwargs.get('upnp_client','') in ('XBox',):
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
             if content_format == 'video/x-msvideo':
                 content_format = 'video/avi'
-            if kwargs.get('upnp_client','') == 'XBox':
-                """ we don't need the DLNA tags there,
-                    and maybe it irritates that poor thing anyway
-                """
-                additional_info = '*'
+            if content_format == 'audio/x-wav':
+                content_format = 'audio/wav'
+            additional_info = self.get_additional_info(upnp_client=kwargs.get('upnp_client',''))
             root.attrib['protocolInfo'] = ':'.join((protocol,network,content_format,additional_info))
         else:
-            root.attrib['protocolInfo'] = self.protocolInfo
+            protocol,network,content_format,additional_info = self.protocolInfo.split(':')
+            if content_format == 'video/x-msvideo':
+                content_format = 'video/avi'
+            additional_info = self.get_additional_info(upnp_client=kwargs.get('upnp_client',''))
+            root.attrib['protocolInfo'] = ':'.join((protocol,network,content_format,additional_info))
+
         root.text = self.data
 
         if self.bitrate is not None:
@@ -178,9 +237,13 @@ class Resource:
         if self.duration is not None:
             root.attrib['duration'] = self.duration
 
+        if self.resolution is not None:
+            root.attrib['resolution'] = self.resolution
+
         if self.importUri is not None:
             root.attrib['importUri'] = self.importUri
 
+
         return root
 
     def fromElement(self, elt):
@@ -189,6 +252,7 @@ class Resource:
         self.bitrate = elt.attrib.get('bitrate')
         self.size = elt.attrib.get('size')
         self.duration = elt.attrib.get('duration',None)
+        self.resolution = elt.attrib.get('resolution',None)
         self.importUri = elt.attrib.get('importUri',None)
 
     def toString(self,**kwargs):
@@ -202,6 +266,32 @@ class Resource:
         instance.fromElement(elt.getroot())
         return instance
 
+    def transcoded(self,format):
+        protocol,network,content_format,additional_info = self.protocolInfo.split(':')
+        dlna_tags = simple_dlna_tags[:]
+        #dlna_tags[1] = 'DLNA.ORG_OP=00'
+        dlna_tags[2] = 'DLNA.ORG_CI=1'
+        if format == 'mp3':
+            if content_format == 'audio/mpeg':
+                return None
+            content_format='audio/mpeg'
+            dlna_pn = 'DLNA.ORG_PN=MP3'
+        elif format == 'lpcm':
+            dlna_pn = 'DLNA.ORG_PN=LPCM'
+            content_format='audio/L16;rate=44100;channels=2'
+        else:
+            return None
+
+        additional_info = ';'.join([dlna_pn]+dlna_tags)
+        new_protocol_info = ':'.join((protocol,network,content_format,additional_info))
+
+        new_res = Resource(self.data+'/transcoded/%s' % format,
+                        new_protocol_info)
+        new_res.size = None
+        new_res.duration = self.duration
+        new_res.resolution = self.resolution
+        return new_res
+
 class Object(log.Loggable):
     """The root class of the entire content directory class heirachy."""
 
@@ -217,6 +307,9 @@ class Object(log.Loggable):
     album = None
     originalTrackNumber=None
 
+    description = None
+    longDescription = None
+
     refID = None
     server_uuid = None
 
@@ -235,12 +328,20 @@ class Object(log.Loggable):
 
         root = ET.Element(self.elementName)
 
-        if self.id == 1000:
-            root.attrib['id'] = '0'
-            ET.SubElement(root, 'dc:title').text = 'root'
-        else:
-            root.attrib['id'] = str(self.id)
-            ET.SubElement(root, 'dc:title').text = self.title
+        #if self.id == 1000:
+        #    root.attrib['id'] = '0'
+        #    ET.SubElement(root, 'dc:title').text = 'root'
+        #else:
+        #    root.attrib['id'] = str(self.id)
+        #    ET.SubElement(root, 'dc:title').text = self.title
+
+        root.attrib['id'] = str(self.id)
+        ET.SubElement(root, 'dc:title').text = self.title
+
+        #if self.title != None:
+        #    ET.SubElement(root, 'dc:title').text = self.title
+        #else:
+        #    ET.SubElement(root, 'dc:title').text = 'root'
 
         root.attrib['parentID'] = str(self.parentID)
 
@@ -249,6 +350,10 @@ class Object(log.Loggable):
                 root.attrib['refID'] = str(self.refID)
 
         if kwargs.get('requested_id',None):
+            if kwargs.get('requested_id') == '0':
+                t = root.find('dc:title')
+                t.text = 'root'
+            #if kwargs.get('requested_id') != '0' and kwargs.get('requested_id') != root.attrib['id']:
             if kwargs.get('requested_id') != root.attrib['id']:
                 if(kwargs.get('upnp_client','') != 'XBox'):
                     root.attrib['refID'] = root.attrib['id']
@@ -275,10 +380,17 @@ class Object(log.Loggable):
                 else:
                     self.info("Changing ID from %r to %r, with parentID from %r to %r", self.id, root.attrib['id'], self.parentID, root.attrib['parentID'])
 
-        if(isinstance(self, Container) and kwargs.get('upnp_client','') == 'XBox'):
-            ET.SubElement(root, 'upnp:class').text = 'object.container.storageFolder'
-        else:
-            ET.SubElement(root, 'upnp:class').text = self.upnp_class
+        ET.SubElement(root, 'upnp:class').text = self.upnp_class
+
+        if kwargs.get('upnp_client','') == 'XBox':
+            u = root.find('upnp:class')
+            if(kwargs.get('parent_container',None) != None and
+                u.text.startswith('object.container')):
+                if kwargs.get('parent_container') in ('14','15','16'):
+                    u.text = 'object.container.storageFolder'
+            if self.upnp_class == 'object.container':
+                u.text = 'object.container.storageFolder'
+
 
         if self.restricted:
             root.attrib['restricted'] = '1'
@@ -311,6 +423,12 @@ class Object(log.Loggable):
         if self.originalTrackNumber is not None:
             ET.SubElement(root, 'upnp:originalTrackNumber').text = str(self.originalTrackNumber)
 
+        if self.description is not None:
+            ET.SubElement(root, 'dc:description').text = self.description
+
+        if self.longDescription is not None:
+            ET.SubElement(root, 'upnp:longDescription').text = self.longDescription
+
         if self.server_uuid is not None:
             ET.SubElement(root, 'upnp:server_uuid').text = self.server_uuid
 
@@ -343,6 +461,10 @@ class Object(log.Loggable):
                 self.albumArtURI = child.text
             elif child.tag.endswith('originalTrackNumber'):
                 self.originalTrackNumber = int(child.text)
+            elif child.tag.endswith('description'):
+                self.description = child.text
+            elif child.tag.endswith('longDescription'):
+                self.longDescription = child.text
             elif child.tag.endswith('artist'):
                 self.artist = child.text
             elif child.tag.endswith('album'):
@@ -370,6 +492,8 @@ class Item(Object):
     elementName = 'item'
     refID = None
 
+    director = None
+
     def __init__(self, *args, **kwargs):
         Object.__init__(self, *args, **kwargs)
         self.res = Resources()
@@ -378,11 +502,34 @@ class Item(Object):
 
         root = Object.toElement(self,**kwargs)
 
+        if self.director is not None:
+            ET.SubElement(root, 'upnp:director').text = self.director
+
         if self.refID is not None:
             ET.SubElement(root, 'refID').text = self.refID
 
-        for res in self.res:
-            root.append(res.toElement(**kwargs))
+        if kwargs.get('transcoding',False) == True:
+            res = self.res.get_matching(['*:*:*:*'], protocol_type='http-get')
+            if len(res) > 0 and is_audio(res[0].protocolInfo):
+                old_res = res[0]
+                if(kwargs.get('upnp_client','') == 'XBox'):
+                    transcoded_res = old_res.transcoded('mp3')
+                    if transcoded_res != None:
+                        root.append(transcoded_res.toElement(**kwargs))
+                    else:
+                        root.append(old_res.toElement(**kwargs))
+                else:
+                    for res in self.res:
+                        root.append(res.toElement(**kwargs))
+                    transcoded_res = old_res.transcoded('lpcm')
+                    if transcoded_res != None:
+                        root.append(transcoded_res.toElement(**kwargs))
+            else:
+                for res in self.res:
+                    root.append(res.toElement(**kwargs))
+        else:
+            for res in self.res:
+                root.append(res.toElement(**kwargs))
 
         return root
 
@@ -391,6 +538,8 @@ class Item(Object):
         for child in elt.getchildren():
             if child.tag.endswith('refID'):
                 self.refID = child.text
+            elif child.tag.endswith('director'):
+                self.director = child.text
             elif child.tag.endswith('res'):
                 res = Resource.fromString(ET.tostring(child))
                 self.res.append(res)
@@ -398,8 +547,6 @@ class Item(Object):
 class ImageItem(Item):
     upnp_class = Item.upnp_class + '.imageItem'
 
-    description = None
-    longDescription = None
     rating = None
     storageMedium = None
     publisher = None
@@ -407,11 +554,6 @@ class ImageItem(Item):
 
     def toElement(self,**kwargs):
         root = Item.toElement(self,**kwargs)
-        if self.description is not None:
-            ET.SubElement(root, 'dc:description').text = self.description
-
-        if self.longDescription is not None:
-            ET.SubElement(root, 'upnp:longDescription').text = self.longDescription
 
         if self.rating is not None:
             ET.SubElement(root, 'upnp:rating').text = str(self.rating)
@@ -443,15 +585,13 @@ class AudioItem(Item):
     upnp_class = Item.upnp_class + '.audioItem'
 
     genre = None
-    description = None
-    longDescription = None
     publisher = None
     language = None
     relation = None
     rights = None
 
     valid_keys = ['genre', 'description', 'longDescription', 'publisher',
-                  'langugage', 'relation', 'rights', 'albumArtURI']
+                  'language', 'relation', 'rights', 'albumArtURI']
 
     #@dlna.AudioItem
     def toElement(self,**kwargs):
@@ -461,13 +601,6 @@ class AudioItem(Item):
         if self.genre is not None:
             ET.SubElement(root, 'upnp:genre').text = self.genre
 
-        if self.description is not None:
-            ET.SubElement(root, 'dc:description').text = self.description
-
-        if self.longDescription is not None:
-            ET.SubElement(root, 'upnp:longDescription').text = \
-                             self.longDescription
-
         if self.publisher is not None:
             ET.SubElement(root, 'dc:publisher').text = self.publisher
 
@@ -497,7 +630,6 @@ class MusicTrack(AudioItem):
     upnp_class = AudioItem.upnp_class + '.musicTrack'
 
     album = None
-    originalTrackNumber = None
     playlist = None
     storageMedium = None
     contributor = None
@@ -509,10 +641,6 @@ class MusicTrack(AudioItem):
         if self.album is not None:
             ET.SubElement(root, 'upnp:album').text = self.album
 
-        if self.originalTrackNumber is not None:
-            ET.SubElement(root, 'upnp:originalTrackNumber').text = \
-                             str(self.originalTrackNumber)
-
         if self.playlist is not None:
             ET.SubElement(root, 'upnp:playlist').text = self.playlist
 
@@ -606,44 +734,76 @@ class Container(Object):
 class Person(Container):
     upnp_class = Container.upnp_class + '.person'
 
+
 class MusicArtist(Person):
     upnp_class = Person.upnp_class + '.musicArtist'
 
+
 class PlaylistContainer(Container):
     upnp_class = Container.upnp_class + '.playlistContainer'
 
+    def __init__(self, id=None, parentID=None, title=None,
+                 restricted = False, creator = None):
+        Container.__init__(self, id, parentID, title, restricted, creator)
+        self.res = Resources()
+
+    def toElement(self,**kwargs):
+        root = Container.toElement(self,**kwargs)
+        for res in self.res:
+            root.append(res.toElement(**kwargs))
+        return root
+
+    def fromElement(self, elt):
+        Container.fromElement(self, elt)
+        for child in elt.getchildren():
+            if child.tag.endswith('res'):
+                res = Resource.fromString(ET.tostring(child))
+                self.res.append(res)
+
+
 class Album(Container):
     upnp_class = Container.upnp_class + '.album'
 
+
 class MusicAlbum(Album):
     upnp_class = Album.upnp_class + '.musicAlbum'
 
+
 class PhotoAlbum(Album):
     upnp_class = Album.upnp_class + '.photoAlbum'
 
+
 class Genre(Container):
     upnp_class = Container.upnp_class + '.genre'
 
+
 class MusicGenre(Genre):
     upnp_class = Genre.upnp_class + '.musicGenre'
 
+
 class MovieGenre(Genre):
     upnp_class = Genre.upnp_class + '.movieGenre'
 
+
 class StorageSystem(Container):
     upnp_class = Container.upnp_class + '.storageSystem'
 
+
 class StorageVolume(Container):
     upnp_class = Container.upnp_class + '.storageVolume'
 
+
 class StorageFolder(Container):
     upnp_class = Container.upnp_class + '.storageFolder'
 
+
 class DIDLElement(ElementInterface,log.Loggable):
 
     logCategory = 'didllite'
 
-    def __init__(self, upnp_client='', parent_container=None, requested_id=None):
+    def __init__(self, upnp_client='',
+                 parent_container=None,requested_id=None,
+                 transcoding=False):
         ElementInterface.__init__(self, 'DIDL-Lite', {})
         self.attrib['xmlns'] = 'urn:schemas-upnp-org:metadata-1-0/DIDL-Lite/'
         self.attrib['xmlns:dc'] = 'http://purl.org/dc/elements/1.1/'
@@ -654,6 +814,7 @@ class DIDLElement(ElementInterface,log.Loggable):
         self.upnp_client = upnp_client
         self.parent_container = parent_container
         self.requested_id = requested_id
+        self.transcoding = transcoding
 
 
     def addContainer(self, id, parentID, title, restricted = False):
@@ -663,7 +824,8 @@ class DIDLElement(ElementInterface,log.Loggable):
     def addItem(self, item):
         self.append(item.toElement(upnp_client=self.upnp_client,
                                    parent_container=self.parent_container,
-                                   requested_id=self.requested_id))
+                                   requested_id=self.requested_id,
+                                   transcoding=self.transcoding))
         self._items.append(item)
 
     def numItems(self):
@@ -703,7 +865,7 @@ class DIDLElement(ElementInterface,log.Loggable):
         elt = elt.getroot()
         for node in elt.getchildren():
             upnp_class_name =  node.findtext('{%s}class' % 'urn:schemas-upnp-org:metadata-1-0/upnp/')
-            upnp_class = instance.get_upnp_class(upnp_class_name)
+            upnp_class = instance.get_upnp_class(upnp_class_name.strip())
             new_node = upnp_class.fromString(ET.tostring(node))
             instance.addItem(new_node)
         return instance
