@@ -43,14 +43,14 @@ class Service(log.Loggable):
 
     def __init__(self, service_type, service_id, location, control_url,
                  event_sub_url, presentation_url, scpd_url, device):
-        if not control_url.startswith('/'):
-            control_url = "/%s" % control_url
-        if not event_sub_url.startswith('/'):
-            event_sub_url = "/%s" % event_sub_url
-        if presentation_url and not presentation_url.startswith('/'):
-            presentation_url = "/%s" % presentation_url
-        if not scpd_url.startswith('/'):
-            scpd_url = "/%s" % scpd_url
+        #if not control_url.startswith('/'):
+        #    control_url = "/%s" % control_url
+        #if not event_sub_url.startswith('/'):
+        #    event_sub_url = "/%s" % event_sub_url
+        #if presentation_url and not presentation_url.startswith('/'):
+        #    presentation_url = "/%s" % presentation_url
+        #if not scpd_url.startswith('/'):
+        #    scpd_url = "/%s" % scpd_url
 
         self.service_type = service_type
         self.detection_completed = False
@@ -77,6 +77,55 @@ class Service(log.Loggable):
         self.parse_actions()
         self.info("%s %s %s initialized" % (self.device.friendly_name,self.service_type,self.id))
 
+    def as_tuples(self):
+        r = []
+
+        def append(name,attribute):
+            try:
+                if isinstance(attribute,tuple):
+                    if callable(attribute[0]):
+                        v1 = attribute[0]()
+                    elif hasattr(self,attribute[0]):
+                        v1 = getattr(self,attribute[0])
+                    else:
+                        v1 = attribute[0]
+                    if v1 in [None,'None']:
+                        return
+                    if callable(attribute[1]):
+                        v2 = attribute[1]()
+                    elif hasattr(self,attribute[1]):
+                        v2 = getattr(self,attribute[1])
+                    else:
+                        v2 = attribute[1]
+                    if v2 in [None,'None']:
+                        return
+                    if len(attribute)> 2:
+                        r.append((name,(v1,v2,attribute[2])))
+                    else:
+                        r.append((name,(v1,v2)))
+                    return
+                elif callable(attribute):
+                    v = attribute()
+                elif hasattr(self,attribute):
+                    v = getattr(self,attribute)
+                else:
+                    v = attribute
+                if v not in [None,'None']:
+                    r.append((name,v))
+            except:
+                import traceback
+                self.debug(traceback.format_exc())
+
+        r.append(('Location',(self.device.get_location(),self.device.get_location())))
+        append('URL base',self.device.get_urlbase)
+        r.append(('UDN',self.device.get_id()))
+        r.append(('Type',self.service_type))
+        append('Service Description URL',(self.scpd_url,lambda: self.device.make_fullyqualified(self.scpd_url)))
+        append('Control URL',(self.control_url,lambda: self.device.make_fullyqualified(self.control_url),False))
+        append('Event Subscription URL',(self.event_sub_url,lambda: self.device.make_fullyqualified(self.event_sub_url),False))
+
+        return r
+
     def __repr__(self):
         return "Service %s %s" % (self.service_type,self.id)
 
@@ -163,20 +212,19 @@ class Service(log.Loggable):
         return self._variables.get(int(instance)).get(name)
 
     def get_control_url(self):
-        return self.url_base + self.control_url
+        return self.device.make_fullyqualified(self.control_url)
 
     def get_event_sub_url(self):
-        #return self.url_base + self.event_sub_url
-        return self.event_sub_url
+        return self.device.make_fullyqualified(self.event_sub_url)
 
     def get_presentation_url(self):
-        return self.url_base + self.presentation_url
+        return self.device.make_fullyqualified(self.presentation_url)
 
     def get_scpd_url(self):
-        return self.url_base + self.scpd_url
+        return self.device.make_fullyqualified(self.scpd_url)
 
     def get_base_url(self):
-        return self.url_base
+        return self.device.make_fullyqualified('.')
 
     def subscribe(self):
         self.debug("subscribe %s", self.id)
@@ -215,7 +263,7 @@ class Service(log.Loggable):
         event.subscribe(self)
 
     def process_event(self,event):
-        self.info("process event %r" % self)
+        self.info("process event %r %r" % (self,event))
         for var_name, var_value  in event.items():
             if var_name == 'LastChange':
                 self.info("we have a LastChange event")
@@ -252,9 +300,14 @@ class Service(log.Loggable):
                                     #self.debug("%r update %r %r %r", self,namespace_uri, tag, var.attrib['val'])
                                     self.get_state_variable(tag, instance_id).update(var.attrib['val'])
                                     self.debug("updated 'attributed' var %r", var)
+                louie.send('Coherence.UPnP.DeviceClient.Service.Event.processed',None,self,(var_name,var_value))
             else:
                 self.get_state_variable(var_name, 0).update(var_value)
+                louie.send('Coherence.UPnP.DeviceClient.Service.Event.processed',None,self,(var_name,var_value))
         if self.last_time_updated == None:
+            # The clients (e.g. media_server_client) check for last time to detect whether service detection is complete
+            # so we need to set it here and now to avoid a potential race condition
+            self.last_time_updated = time.time()
             louie.send('Coherence.UPnP.DeviceClient.Service.notified', sender=self.device, service=self)
             self.info("send signal Coherence.UPnP.DeviceClient.Service.notified for %r" % self)
         self.last_time_updated = time.time()
@@ -288,18 +341,22 @@ class Service(log.Loggable):
                     {urn:schemas-beebits-net:service-1-0}X_withVendorDefines
                     attibute there
                 """
-                #for allowed in var_node.findall('.//{%s}allowedValue' % ns):
-                #    values.append(allowed.text)
+                for allowed in var_node.findall('.//{%s}allowedValue' % ns):
+                    values.append(allowed.text)
                 instance = 0
                 self._variables.get(instance)[name] = variable.StateVariable(self, name,
                                                                'n/a',
                                                                instance, send_events,
                                                                data_type, values)
+                """ we need to do this here, as there we don't get there our
+                    {urn:schemas-beebits-net:service-1-0}X_withVendorDefines
+                    attibute there
+                """
+                self._variables.get(instance)[name].has_vendor_values = True
 
 
             #print 'service parse:', self, self.device
             self.detection_completed = True
-
             louie.send('Coherence.UPnP.Service.detection_completed', sender=self.device, device=self.device)
             self.info("send signal Coherence.UPnP.Service.detection_completed for %r" % self)
             """
@@ -788,6 +845,7 @@ class scpdXML(static.Data):
         self.xml = """<?xml version="1.0" encoding="utf-8"?>""" + ET.tostring( root, encoding='utf-8')
         static.Data.__init__(self, self.xml, 'text/xml')
 
+from twisted.python.util import OrderedDict
 
 class ServiceControl:
 
@@ -820,8 +878,14 @@ class ServiceControl:
                     #print "r", r
             self.service.propagate_notification(notify)
         #r= { '%sResponse'%action.name: r}
-        self.info( 'action_results', action.name, r)
-        return r
+        self.info( 'action_results unsorted', action.name, r)
+        if len(r) == 0:
+            return r
+        ordered_result = OrderedDict()
+        for argument in action.get_out_arguments():
+            ordered_result[argument.name] = r[argument.name]
+        self.info( 'action_results sorted', action.name, ordered_result)
+        return ordered_result
 
     def soap__generic(self, *args, **kwargs):
         """ generic UPnP service control method,
