@@ -14,9 +14,12 @@ from coherence import SERVER_ID
 from twisted.web import server, http, static
 from twisted.web import client, error
 from twisted.web import proxy, resource, server
-from twisted.internet import reactor,protocol,defer
+from twisted.internet import reactor,protocol,defer,abstract
 from twisted.python import failure
 
+from twisted.python.util import InsensitiveDict
+
+
 try:
     from twisted.protocols._c_urlarg import unquote
 except ImportError:
@@ -28,6 +31,25 @@ try:
 except ImportError:
     have_netifaces = False
 
+
+def means_true(value):
+    if isinstance(value,basestring):
+        value = value.lower()
+    return value in [True,1,'1','true','yes','ok']
+
+def generalise_boolean(value):
+    """ standardize the different boolean incarnations
+
+        transform anything that looks like a "True" into a '1',
+        and everything else into a '0'
+    """
+    if means_true(value):
+        return '1'
+    return '0'
+
+generalize_boolean = generalise_boolean
+
+
 def parse_xml(data, encoding="utf-8"):
     return et_parse_xml(data,encoding)
 
@@ -72,7 +94,7 @@ def get_ip_address(ifname):
         if have_netifaces:
             if ifname in netifaces.interfaces():
                 iface = netifaces.ifaddresses(ifname)
-                ifaceadr = iface(netifaces.AF_INET)
+                ifaceadr = iface[netifaces.AF_INET]
                 # we now have a list of address dictionaries, there may be multiple addresses bound
                 return ifaceadr[0]['addr']
         return '127.0.0.1'
@@ -90,11 +112,15 @@ def get_ip_address(ifname):
         SIOCGIFADDR = 0xc0206921
 
     s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-    return socket.inet_ntoa(fcntl.ioctl(
-        s.fileno(),
-        SIOCGIFADDR,
-        struct.pack('256s', ifname[:15])
-    )[20:24])
+    try:
+        return socket.inet_ntoa(fcntl.ioctl(
+            s.fileno(),
+            SIOCGIFADDR,
+            struct.pack('256s', ifname[:15])
+        )[20:24])
+    except IOError:
+        return '127.0.0.1'
+
 
 def get_host_address():
     """ try to get determine the interface used for
@@ -105,46 +131,51 @@ def get_host_address():
     import sys
     if sys.platform == 'win32':
         if have_netifaces:
-            return get_ip_address(netifaces.interfaces()[0])    # on windows assume first interface is primary
-
-    try:
-        route_file = '/proc/net/route'
-        route = open(route_file)
-        if(route):
-            tmp = route.readline() #skip first line
-            while (tmp != ''):
-                tmp = route.readline()
-                l = tmp.split('\t')
-                if (len(l) > 2):
-                    if l[2] != '00000000': #default gateway...
-                        route.close()
-                        return get_ip_address(l[0])
-    except IOerror, msg:
-        """ fallback to parsing the output of netstat """
-        from os import uname
-        import posix
-        (osname,_, _, _,_) = uname()
-        osname = osname.lower()
-        f = posix.popen('netstat -rn')
-        lines = f.readlines()
-        f.close()
-        for l in lines:
-            parts = [x.strip() for x in l.split(' ') if len(x) > 0]
-            if parts[0] in ('0.0.0.0','default'):
-                if osname[:6] == 'darwin':
-                    return get_ip_address(parts[5])
-                else:
-                    return get_ip_address(parts[-1])
-    except Exception, msg:
-        import traceback
-        traceback.print_exc()
+            interfaces = netifaces.interfaces()
+            if len(interfaces):
+                return get_ip_address(interfaces[0])    # on windows assume first interface is primary
+    else:
+        try:
+            route_file = '/proc/net/route'
+            route = open(route_file)
+            if(route):
+                tmp = route.readline() #skip first line
+                while (tmp != ''):
+                    tmp = route.readline()
+                    l = tmp.split('\t')
+                    if (len(l) > 2):
+                        if l[1] == '00000000': #default route...
+                            route.close()
+                            return get_ip_address(l[0])
+        except IOError, msg:
+            """ fallback to parsing the output of netstat """
+            from os import uname
+            import posix
+            (osname,_, _, _,_) = uname()
+            osname = osname.lower()
+            f = posix.popen('netstat -rn')
+            lines = f.readlines()
+            f.close()
+            for l in lines:
+                parts = [x.strip() for x in l.split(' ') if len(x) > 0]
+                if parts[0] in ('0.0.0.0','default'):
+                    if osname[:6] == 'darwin':
+                        return get_ip_address(parts[5])
+                    else:
+                        return get_ip_address(parts[-1])
+        except Exception, msg:
+            import traceback
+            traceback.print_exc()
 
     """ return localhost if we haven't found anything """
     return '127.0.0.1'
 
 def de_chunk_payload(response):
 
-    import StringIO
+    try:
+        import cStringIO as StringIO
+    except ImportError:
+        import StringIO
     """ This method takes a chunked HTTP data object and unchunks it."""
     newresponse = StringIO.StringIO()
     # chunked encoding consists of a bunch of lines with
@@ -218,9 +249,9 @@ class ProxyClient(http.HTTPClient):
             del headers["proxy-connection"]
         #headers["connection"] = "close"
         self.headers = headers
-        print "command", command
-        print "rest", rest
-        print "headers", headers
+        #print "command", command
+        #print "rest", rest
+        #print "headers", headers
         self.data = data
         self.send_data = 0
 
@@ -239,13 +270,13 @@ class ProxyClient(http.HTTPClient):
 
         if version == 'ICY':
             version = 'HTTP/1.1'
-        print "ProxyClient handleStatus", version, code, message
+        #print "ProxyClient handleStatus", version, code, message
         self.father.transport.write("%s %s %s\r\n" % (version, code, message))
 
     def handleHeader(self, key, value):
         #print "ProxyClient handleHeader", key, value
         if not key.startswith('icy-'):
-            print "ProxyClient handleHeader", key, value
+            #print "ProxyClient handleHeader", key, value
             self.father.transport.write("%s: %s\r\n" % (key, value))
 
     def handleEndHeaders(self):
@@ -333,6 +364,7 @@ class ReverseProxyResource(proxy.ReverseProxyResource):
         self.host = host
         self.port = port
         self.path = path
+        self.qs = ''
         self.reactor = reactor
 
     def getChild(self, path, request):
@@ -351,6 +383,8 @@ class ReverseProxyResource(proxy.ReverseProxyResource):
             request.received_headers['host'] = "%s:%d" % (self.host, self.port)
         request.content.seek(0, 0)
         qs = urlparse.urlparse(request.uri)[4]
+        if qs == '':
+            qs = self.qs
         if qs:
             rest = self.path + '?' + qs
         else:
@@ -361,9 +395,41 @@ class ReverseProxyResource(proxy.ReverseProxyResource):
         self.reactor.connectTCP(self.host, self.port, clientFactory)
         return server.NOT_DONE_YET
 
+    def resetTarget(self,host,port,path,qs=''):
+        self.host = host
+        self.port = port
+        self.path = path
+        self.qs = qs
+
 
 class myHTTPPageGetter(client.HTTPPageGetter):
 
+    followRedirect = True
+
+    def connectionMade(self):
+        method = getattr(self, 'method', 'GET')
+        #print "myHTTPPageGetter", method, self.factory.path
+        self.sendCommand(method, self.factory.path)
+        self.sendHeader('Host', self.factory.headers.get("host", self.factory.host))
+        self.sendHeader('User-Agent', self.factory.agent)
+        if self.factory.cookies:
+            l=[]
+            for cookie, cookval in self.factory.cookies.items():
+                l.append('%s=%s' % (cookie, cookval))
+            self.sendHeader('Cookie', '; '.join(l))
+        data = getattr(self.factory, 'postdata', None)
+        if data is not None:
+            self.sendHeader("Content-Length", str(len(data)))
+        for (key, value) in self.factory.headers.items():
+            if key.lower() != "content-length":
+                # we calculated it on our own
+                self.sendHeader(key, value)
+        self.endHeaders()
+        self.headers = {}
+
+        if data is not None:
+            self.transport.write(data)
+
     def handleResponse(self, response):
         if self.quietLoss:
             return
@@ -393,6 +459,44 @@ class HeaderAwareHTTPClientFactory(client.HTTPClientFactory):
     protocol = myHTTPPageGetter
     noisy = False
 
+    def __init__(self, url, method='GET', postdata=None, headers=None,
+                 agent="Twisted PageGetter", timeout=0, cookies=None,
+                 followRedirect=True, redirectLimit=20):
+        self.followRedirect = followRedirect
+        self.redirectLimit = redirectLimit
+        self._redirectCount = 0
+        self.timeout = timeout
+        self.agent = agent
+
+        if cookies is None:
+            cookies = {}
+        self.cookies = cookies
+        if headers is not None:
+            self.headers = InsensitiveDict(headers)
+        else:
+            self.headers = InsensitiveDict()
+        if postdata is not None:
+            self.headers.setdefault('Content-Length', len(postdata))
+            # just in case a broken http/1.1 decides to keep connection alive
+            self.headers.setdefault("connection", "close")
+        self.postdata = postdata
+        self.method = method
+
+        self.setURL(url)
+
+        self.waiting = 1
+        self.deferred = defer.Deferred()
+        self.response_headers = None
+
+    def buildProtocol(self, addr):
+        p = protocol.ClientFactory.buildProtocol(self, addr)
+        p.method = self.method
+        p.followRedirect = self.followRedirect
+        if self.timeout:
+            timeoutCall = reactor.callLater(self.timeout, p.timeout)
+            self.deferred.addBoth(self._cancelTimeout, timeoutCall)
+        return p
+
     def page(self, page):
         if self.waiting:
             self.waiting = 0
@@ -499,10 +603,8 @@ class StaticFile(static.File):
                 return error.ForbiddenResource().render(request)
             else:
                 raise
-
         if request.setLastModified(self.getmtime()) is http.CACHED:
             return ''
-
         trans = True
 
         range = request.getHeader('range')
@@ -550,6 +652,7 @@ class StaticFile(static.File):
         if request.method == 'HEAD' or trans == False:
             # pretend we're a HEAD request, so content-length
             # won't be overwritten.
+            #print "HEAD request"
             request.method = 'HEAD'
             return ''
 
@@ -561,6 +664,174 @@ class StaticFile(static.File):
         # and make sure the connection doesn't get closed
         return server.NOT_DONE_YET
 
+class BufferFile(static.File):
+    """ taken from twisted.web.static and modified
+        accordingly to the patch by John-Mark Gurney
+        http://resnet.uoregon.edu/~gurney_j/jmpc/dist/twisted.web.static.patch
+    """
+
+    def __init__(self, path, target_size=0, *args):
+        static.File.__init__(self, path, *args)
+        self.target_size = target_size
+        self.upnp_retry = None
+
+    def render(self, request):
+        #print ""
+        #print "BufferFile", request
+
+        # FIXME detect when request is REALLY finished
+        if request is None or request.finished :
+            print "No request to render!"
+            return ''
+
+        """You know what you doing."""
+        self.restat()
+
+        if self.type is None:
+            self.type, self.encoding = static.getTypeAndEncoding(self.basename(),
+                                                          self.contentTypes,
+                                                          self.contentEncodings,
+                                                          self.defaultType)
+
+        if not self.exists():
+            return self.childNotFound.render(request)
+
+        if self.isdir():
+            return self.redirect(request)
+
+        #for content-length
+        if (self.target_size > 0):
+            fsize = size = int(self.target_size)
+        else:
+            fsize = size = int(self.getFileSize())
+
+        #print fsize
+
+        if size == int(self.getFileSize()):
+            request.setHeader('accept-ranges','bytes')
+
+        if self.type:
+            request.setHeader('content-type', self.type)
+        if self.encoding:
+            request.setHeader('content-encoding', self.encoding)
+
+        try:
+            f = self.openForReading()
+        except IOError, e:
+            import errno
+            if e[0] == errno.EACCES:
+                return error.ForbiddenResource().render(request)
+            else:
+                raise
+        if request.setLastModified(self.getmtime()) is http.CACHED:
+            return ''
+        trans = True
+
+        range = request.getHeader('range')
+        #print "StaticFile", range
+
+        tsize = size
+        if range is not None:
+            # This is a request for partial data...
+            bytesrange = range.split('=')
+            assert bytesrange[0] == 'bytes',\
+                   "Syntactically invalid http range header!"
+            start, end = bytesrange[1].split('-', 1)
+            if start:
+                start = int(start)
+                # Are we requesting something beyond the current size of the file?
+                if (start >= self.getFileSize()):
+                    # Retry later!
+                    print bytesrange
+                    print "Requesting data beyond current scope -> postpone rendering!"
+                    self.upnp_retry = reactor.callLater(1.0, self.render, request)
+                    return server.NOT_DONE_YET
+
+                f.seek(start)
+                if end:
+                    #print ":%s" % end
+                    end = int(end)
+                else:
+                    end = size - 1
+            else:
+                lastbytes = int(end)
+                if size < lastbytes:
+                    lastbytes = size
+                start = size - lastbytes
+                f.seek(start)
+                fsize = lastbytes
+                end = size - 1
+            size = end + 1
+            fsize = end - int(start) + 1
+            # start is the byte offset to begin, and end is the byte offset
+            # to end..  fsize is size to send, tsize is the real size of
+            # the file, and size is the byte position to stop sending.
+            if fsize <= 0:
+                request.setResponseCode(http.REQUESTED_RANGE_NOT_SATISFIABLE)
+                fsize = tsize
+                trans = False
+            else:
+                request.setResponseCode(http.PARTIAL_CONTENT)
+                request.setHeader('content-range',"bytes %s-%s/%s " % (
+                    str(start), str(end), str(tsize)))
+                #print "StaticFile", start, end, tsize
+
+        request.setHeader('content-length', str(fsize))
+
+        if request.method == 'HEAD' or trans == False:
+            # pretend we're a HEAD request, so content-length
+            # won't be overwritten.
+            request.method = 'HEAD'
+            return ''
+
+        #print "StaticFile out", request.headers, request.code
+
+        # return data
+        # size is the byte position to stop sending, not how many bytes to send
+
+        BufferFileTransfer(f, size - f.tell(), request)
+        # and make sure the connection doesn't get closed
+        return server.NOT_DONE_YET
+
+
+class BufferFileTransfer(object):
+    """
+    A class to represent the transfer of a file over the network.
+    """
+    request = None
+
+    def __init__(self, file, size, request):
+        self.file = file
+        self.size = size
+        self.request = request
+        self.written = self.file.tell()
+        request.registerProducer(self, 0)
+
+    def resumeProducing(self):
+        #print "resumeProducing", self.request,self.size,self.written
+        if not self.request:
+            return
+        data = self.file.read(min(abstract.FileDescriptor.bufferSize, self.size - self.written))
+        if data:
+            self.written += len(data)
+            # this .write will spin the reactor, calling .doWrite and then
+            # .resumeProducing again, so be prepared for a re-entrant call
+            self.request.write(data)
+        if self.request and self.file.tell() == self.size:
+            self.request.unregisterProducer()
+            self.request.finish()
+            self.request = None
+
+    def pauseProducing(self):
+        pass
+
+    def stopProducing(self):
+        #print "stopProducing",self.request
+        self.request.unregisterProducer()
+        self.file.close()
+        self.request.finish()
+        self.request = None
+
 
 from datetime import datetime, tzinfo, timedelta
 import random
