@@ -9,11 +9,15 @@ import os.path
 
 from twisted.python import util
 from twisted.web import resource, static
+from twisted.internet import reactor
 
 from coherence import __version__
 
 from coherence.extern.et import ET, indent
 
+import coherence.extern.louie as louie
+
+
 from coherence import log
 
 
@@ -68,15 +72,31 @@ class RootDeviceXML(static.Data):
         uuid = str(uuid)
         root = ET.Element('root')
         root.attrib['xmlns']='urn:schemas-upnp-org:device-1-0'
-        device_type = 'urn:schemas-upnp-org:device:%s:%d' % (device_type, version)
+        _device_type = 'urn:schemas-upnp-org:device:%s:%d' % (device_type, version)
         e = ET.SubElement(root, 'specVersion')
         ET.SubElement( e, 'major').text = '1'
         ET.SubElement( e, 'minor').text = '0'
 
-        ET.SubElement(root, 'URLBase').text = urlbase
+        #ET.SubElement(root, 'URLBase').text = urlbase + uuid[5:] + '/'
 
         d = ET.SubElement(root, 'device')
-        ET.SubElement( d, 'deviceType').text = device_type
+
+        if device_type == 'MediaServer':
+            x = ET.SubElement(d, 'dlna:X_DLNADOC')
+            x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+            x.text = 'DMS-1.50'
+            x = ET.SubElement(d, 'dlna:X_DLNADOC')
+            x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+            x.text = 'M-DMS-1.50'
+        elif device_type == 'MediaRenderer':
+            x = ET.SubElement(d, 'dlna:X_DLNADOC')
+            x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+            x.text = 'DMR-1.50'
+            x = ET.SubElement(d, 'dlna:X_DLNADOC')
+            x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+            x.text = 'M-DMR-1.50'
+
+        ET.SubElement( d, 'deviceType').text = _device_type
         ET.SubElement( d, 'friendlyName').text = friendly_name
         ET.SubElement( d, 'manufacturer').text = 'beebits.net'
         ET.SubElement( d, 'manufacturerURL').text = 'http://coherence.beebits.net'
@@ -109,9 +129,9 @@ class RootDeviceXML(static.Data):
                 except:
                     namespace = 'upnp-org'
                 ET.SubElement( s, 'serviceId').text = 'urn:%s:serviceId:%s' % (namespace,id)
-                ET.SubElement( s, 'SCPDURL').text = '/' + uuid[5:] + '/' + id + '/' + service.scpd_url
-                ET.SubElement( s, 'controlURL').text = '/' + uuid[5:] + '/' + id + '/' + service.control_url
-                ET.SubElement( s, 'eventSubURL').text = '/' + uuid[5:] + '/' + id + '/' + service.subscription_url
+                ET.SubElement( s, 'SCPDURL').text = id + '/' + service.scpd_url # '/' + uuid[5:] + '/' +
+                ET.SubElement( s, 'controlURL').text = id + '/' + service.control_url
+                ET.SubElement( s, 'eventSubURL').text = id + '/' + service.subscription_url
 
         if len(devices):
             e = ET.SubElement( d, 'deviceList')
@@ -135,6 +155,48 @@ class RootDeviceXML(static.Data):
 
 class BasicDeviceMixin(object):
 
+    def __init__(self, coherence, backend, **kwargs):
+        self.coherence = coherence
+        if not hasattr(self,'version'):
+            self.version = int(kwargs.get('version',self.coherence.config.get('version',2)))
+
+        try:
+            self.uuid = kwargs['uuid']
+            if not self.uuid.startswith('uuid:'):
+                self.uuid = 'uuid:' + self.uuid
+        except KeyError:
+            from coherence.upnp.core.uuid import UUID
+            self.uuid = UUID()
+
+        self.backend = None
+        urlbase = self.coherence.urlbase
+        if urlbase[-1] != '/':
+            urlbase += '/'
+        self.urlbase = urlbase + str(self.uuid)[5:]
+
+        kwargs['urlbase'] = self.urlbase
+        self.icons = kwargs.get('iconlist', kwargs.get('icons', []))
+        if len(self.icons) == 0:
+            if kwargs.has_key('icon'):
+                if isinstance(kwargs['icon'],dict):
+                    self.icons.append(kwargs['icon'])
+                else:
+                    self.icons = kwargs['icon']
+
+        louie.connect( self.init_complete, 'Coherence.UPnP.Backend.init_completed', louie.Any)
+        louie.connect( self.init_failed, 'Coherence.UPnP.Backend.init_failed', louie.Any)
+        reactor.callLater(0.2, self.fire, backend, **kwargs)
+
+    def init_failed(self, backend, msg):
+        if self.backend != backend:
+            return
+        self.warning('backend not installed, %s activation aborted - %s' % (self.device_type,msg.getErrorMessage()))
+        self.debug(msg)
+        try:
+            del self.coherence.active_backends[str(self.uuid)]
+        except KeyError:
+            pass
+
     def register(self):
         s = self.coherence.ssdp_server
         uuid = str(self.uuid)
@@ -181,10 +243,14 @@ class BasicDeviceMixin(object):
                 except:
                     namespace = 'schemas-upnp-org'
 
+                device_description_tmpl = 'description-%d.xml'  % device_version
+                if hasattr(service,'device_description_tmpl'):
+                    device_description_tmpl = service.device_description_tmpl
+
                 s.register('local',
                             '%s::urn:%s:service:%s:%d' % (uuid,namespace,service.id, service_version),
                             'urn:%s:service:%s:%d' % (namespace,service.id, service_version),
-                            self.coherence.urlbase + uuid[5:] + '/' + 'description-%d.xml' % device_version,
+                            self.coherence.urlbase + uuid[5:] + '/' + device_description_tmpl,
                             silent=silent,
                             host=host)
 
