@@ -12,7 +12,7 @@ from StringIO import StringIO
 import urllib
 
 from twisted.internet import task
-from twisted.internet import reactor, defer
+from twisted.internet import defer
 from twisted.web import static
 from twisted.web import resource, server
 #from twisted.web import proxy
@@ -34,14 +34,14 @@ from coherence.upnp.services.servers.media_receiver_registrar_server import Fake
 
 from coherence.upnp.devices.basics import BasicDeviceMixin
 
-import coherence.extern.louie as louie
-
 from coherence import log
 
 COVER_REQUEST_INDICATOR = re.compile(".*?cover\.[A-Z|a-z]{3,4}$")
 
 ATTACHMENT_REQUEST_INDICATOR = re.compile(".*?attachment=.*$")
 
+TRANSCODED_REQUEST_INDICATOR = re.compile(".*/transcoded/.*$")
+
 class MSRoot(resource.Resource, log.Loggable):
     logCategory = 'mediaserver'
 
@@ -68,7 +68,11 @@ class MSRoot(resource.Resource, log.Loggable):
         headers = request.getAllHeaders()
         self.msg( request.getAllHeaders())
 
-        if request.method == 'GET':
+        try:
+            request._dlna_transfermode = headers['transfermode.dlna.org']
+        except KeyError:
+            request._dlna_transfermode = 'Streaming'
+        if request.method in ('GET','HEAD'):
             if COVER_REQUEST_INDICATOR.match(request.uri):
                 self.info("request cover for id %s" % path)
                 ch = self.store.get_by_id(path)
@@ -85,15 +89,98 @@ class MSRoot(resource.Resource, log.Loggable):
                 self.info("request attachment %r for id %s" % (request.args,path))
                 ch = self.store.get_by_id(path)
                 try:
-                    return ch.item.attachments[request.args['attachment'][0]]
+                    #FIXME same as below
+                    if 'transcoded' in request.args:
+                        if self.server.coherence.config.get('transcoding', 'no') == 'yes':
+                            format = request.args['transcoded'][0]
+                            type = request.args['type'][0]
+                            self.info("request transcoding %r %r" % (format, type))
+                            if format == 'thumb' and type == 'jpeg':
+                                try:
+                                    from coherence.transcoder import JPEGThumbTranscoder
+                                    return JPEGThumbTranscoder(ch.item.attachments[request.args['attachment'][0]])
+                                except:
+                                    self.debug(traceback.format_exc())
+                            request.setResponseCode(404)
+                            return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                        else:
+                            request.setResponseCode(404)
+                            return static.Data("<html><p>This MediaServer doesn't support transcoding</p></html>",'text/html')
+                    else:
+                        return ch.item.attachments[request.args['attachment'][0]]
                 except:
                     request.setResponseCode(404)
                     return static.Data('<html><p>the requested attachment was not found</p></html>','text/html')
 
+        #if(request.method in ('GET','HEAD') and
+        #   XBOX_TRANSCODED_REQUEST_INDICATOR.match(request.uri)):
+        #    if self.server.coherence.config.get('transcoding', 'no') == 'yes':
+        #        id = path[:-15].split('/')[-1]
+        #        self.info("request transcoding to %r for id %s" % (request.args,id))
+        #        ch = self.store.get_by_id(id)
+        #        uri = ch.get_path()
+        #        return MP3Transcoder(uri)
+
+        if(request.method in ('GET','HEAD') and
+           TRANSCODED_REQUEST_INDICATOR.match(request.uri)):
+            self.info("request transcoding to %s for id %s" % (request.uri.split('/')[-1],path))
+            if self.server.coherence.config.get('transcoding', 'no') == 'yes':
+                ch = self.store.get_by_id(path)
+                #FIXME create a generic transcoder class and sort the details there
+                format = request.uri.split('/')[-1] #request.args['transcoded'][0]
+                uri = ch.get_path()
+                if format == 'lpcm':
+                    try:
+                        from coherence.transcoder import PCMTranscoder
+                        return PCMTranscoder(uri)
+                    except:
+                        self.debug(traceback.format_exc())
+                        request.setResponseCode(404)
+                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                if format == 'wav':
+                    try:
+                        from coherence.transcoder import WAVTranscoder
+                        return WAVTranscoder(uri)
+                    except:
+                        self.debug(traceback.format_exc())
+                        request.setResponseCode(404)
+                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                if format == 'mp3':
+                    try:
+                        from coherence.transcoder import MP3Transcoder
+                        return MP3Transcoder(uri)
+                    except:
+                        self.debug(traceback.format_exc())
+                        request.setResponseCode(404)
+                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                if format == 'mp4':
+                    try:
+                        from coherence.transcoder import MP4Transcoder
+                        return MP4Transcoder(uri)
+                    except:
+                        self.debug(traceback.format_exc())
+                        request.setResponseCode(404)
+                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                if format == 'thumb':
+                    type = request.args['type'][0]
+                    if type == 'jpeg':
+                        try:
+                            from coherence.transcoder import JPEGThumbTranscoder
+                            return JPEGThumbTranscoder(uri)
+                        except:
+                            self.debug(traceback.format_exc())
+                            request.setResponseCode(404)
+                            return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+            request.setResponseCode(404)
+            return static.Data("<html><p>This MediaServer doesn't support transcoding</p></html>",'text/html')
+
         if(request.method == 'POST' and
            request.uri.endswith('?import')):
-            self.import_file(path,request)
-            return self.import_response(path)
+            d = self.import_file(path,request)
+            if isinstance(d, defer.Deferred):
+                d.addBoth(self.import_response,path)
+                return d
+            return self.import_response(None,path)
 
         if(headers.has_key('user-agent') and
            (headers['user-agent'].find('Xbox/') == 0 or      # XBox
@@ -118,16 +205,16 @@ class MSRoot(resource.Resource, log.Loggable):
 
     def import_file(self,name,request):
         self.info("import file, id %s" % name)
+        print "import file, id %s" % name
         ch = self.store.get_by_id(name)
+        print "ch", ch
         if ch is not None:
-            try:
-                f = open(ch.get_path(), 'w+b')
-                f.write(request.content.read()) #FIXME: is this the right way?
-                f.close()
-                request.setResponseCode(200)
+            if hasattr(self.store,'backend_import'):
+                response_code = self.store.backend_import(ch,request.content)
+                if isinstance(response_code, defer.Deferred):
+                    return response_code
+                request.setResponseCode(response_code)
                 return
-            except IOError:
-                self.warning("import of file %s failed" % ch.get_path())
 
         request.setResponseCode(404)
 
@@ -162,10 +249,10 @@ class MSRoot(resource.Resource, log.Loggable):
                     self.info("startup, add %d to connection table" % new_id)
                     d = request.notifyFinish()
                     d.addBoth(self.requestFinished, new_id, request)
-                    request.setHeader('transferMode.dlna.org', 'Streaming')
+                    request.setHeader('transferMode.dlna.org', request._dlna_transfermode)
                     if hasattr(ch,'item') and hasattr(ch.item, 'res'):
                         if ch.item.res[0].protocolInfo is not None:
-                            _,_,_,additional_info = ch.item.res[0].protocolInfo.split(':')
+                            additional_info = ch.item.res[0].get_additional_info()
                             if additional_info != '*':
                                 request.setHeader('contentFeatures.dlna.org', additional_info)
                     return ch.location
@@ -186,10 +273,10 @@ class MSRoot(resource.Resource, log.Loggable):
                 self.info("startup, add %d to connection table" % new_id)
                 d = request.notifyFinish()
                 d.addBoth(self.requestFinished, new_id, request)
-                request.setHeader('transferMode.dlna.org', 'Streaming')
+                request.setHeader('transferMode.dlna.org', request._dlna_transfermode)
                 if hasattr(ch, 'item') and hasattr(ch.item, 'res'):
                     if ch.item.res[0].protocolInfo is not None:
-                        _,_,_,additional_info = ch.item.res[0].protocolInfo.split(':')
+                        additional_info = ch.item.res[0].get_additional_info()
                         if additional_info != '*':
                             request.setHeader('contentFeatures.dlna.org', additional_info)
                 ch = StaticFile(p)
@@ -225,36 +312,37 @@ class MSRoot(resource.Resource, log.Loggable):
                 uri += '/'
 
             def build_page(r,page):
-                print "build_page", r
+                #print "build_page", r
                 page += """<ul>"""
-                for c in r:
-                    if hasattr(c,'get_url'):
-                        path = c.get_url()
-                        self.debug('has get_url', path)
-                    elif hasattr(c,'get_path') and c.get_path != None:
-                        #path = c.get_path().encode('utf-8').encode('string_escape')
-                        path = c.get_path()
-                        if isinstance(path,unicode):
-                            path = path.encode('ascii','xmlcharrefreplace')
+                if r is not None:
+                    for c in r:
+                        if hasattr(c,'get_url'):
+                            path = c.get_url()
+                            self.debug('has get_url', path)
+                        elif hasattr(c,'get_path') and c.get_path != None:
+                            #path = c.get_path().encode('utf-8').encode('string_escape')
+                            path = c.get_path()
+                            if isinstance(path,unicode):
+                                path = path.encode('ascii','xmlcharrefreplace')
+                            else:
+                                path = path.decode('utf-8').encode('ascii','xmlcharrefreplace')
+                            self.debug('has get_path', path)
                         else:
-                            path = path.decode('utf-8').encode('ascii','xmlcharrefreplace')
-                        self.debug('has get_path', path)
-                    else:
-                        path = request.uri.split('/')
-                        path[-1] = str(c.get_id())
-                        path = '/'.join(path)
-                        self.debug('got path', path)
-                    title = c.get_name()
-                    self.debug( 'title is:', type(title))
-                    try:
-                        if isinstance(title,unicode):
-                            title = title.encode('ascii','xmlcharrefreplace')
-                        else:
-                            title = title.decode('utf-8').encode('ascii','xmlcharrefreplace')
-                    except (UnicodeEncodeError,UnicodeDecodeError):
-                        title = c.get_name().encode('utf-8').encode('string_escape')
-                    page += '<li><a href="%s">%s</a></li>' % \
-                                        (path, title)
+                            path = request.uri.split('/')
+                            path[-1] = str(c.get_id())
+                            path = '/'.join(path)
+                            self.debug('got path', path)
+                        title = c.get_name()
+                        self.debug( 'title is:', type(title))
+                        try:
+                            if isinstance(title,unicode):
+                                title = title.encode('ascii','xmlcharrefreplace')
+                            else:
+                                title = title.decode('utf-8').encode('ascii','xmlcharrefreplace')
+                        except (UnicodeEncodeError,UnicodeDecodeError):
+                            title = c.get_name().encode('utf-8').encode('string_escape')
+                        page += '<li><a href="%s">%s</a></li>' % \
+                                            (path, title)
                 page += """</ul>"""
                 page += """</body></html>"""
                 return static.Data(page,'text/html')
@@ -288,7 +376,7 @@ class MSRoot(resource.Resource, log.Loggable):
                 cl += '<li><a href=%s%s>%s</a></li>' % (uri,c,c)
         return cl
 
-    def import_response(self,id):
+    def import_response(self,result,id):
         return static.Data('<html><p>import of %s finished</p></html>'% id,'text/html')
 
     def render(self,request):
@@ -316,7 +404,8 @@ class RootDeviceXML(static.Data):
         ET.SubElement(e, 'major').text = '1'
         ET.SubElement(e, 'minor').text = '0'
 
-        ET.SubElement(root, 'URLBase').text = urlbase
+        #if version == 1:
+        #    ET.SubElement(root, 'URLBase').text = urlbase + uuid[5:] + '/'
 
         d = ET.SubElement(root, 'device')
         x = ET.SubElement(d, 'dlna:X_DLNADOC')
@@ -343,12 +432,14 @@ class RootDeviceXML(static.Data):
         ET.SubElement(d, 'serialNumber').text = '0000001'
         ET.SubElement(d, 'UDN').text = uuid
         ET.SubElement(d, 'UPC').text = ''
-        ET.SubElement(d, 'presentationURL').text = ''
+        ET.SubElement(d, 'presentationURL').text = '/' + uuid[5:]
 
         if len(services):
             e = ET.SubElement(d, 'serviceList')
             for service in services:
                 id = service.get_id()
+                if xbox_hack == False and id == 'X_MS_MediaReceiverRegistrar':
+                    continue
                 s = ET.SubElement(e, 'service')
                 try:
                     namespace = service.namespace
@@ -369,6 +460,10 @@ class RootDeviceXML(static.Data):
                 ET.SubElement(s, 'controlURL').text = '/' + uuid[5:] + '/' + id + '/' + service.control_url
                 ET.SubElement(s, 'eventSubURL').text = '/' + uuid[5:] + '/' + id + '/' + service.subscription_url
 
+                #ET.SubElement(s, 'SCPDURL').text = id + '/' + service.scpd_url
+                #ET.SubElement(s, 'controlURL').text = id + '/' + service.control_url
+                #ET.SubElement(s, 'eventSubURL').text = id + '/' + service.subscription_url
+
         if len(devices):
             e = ET.SubElement(d, 'deviceList')
 
@@ -381,6 +476,12 @@ class RootDeviceXML(static.Data):
                         if v.startswith('file://'):
                             ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+os.path.basename(v)
                             continue
+                        elif v == '.face':
+                            ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+'face-icon.png'
+                            continue
+                        else:
+                            ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+os.path.basename(v)
+                            continue
                     ET.SubElement(i, k).text = str(v)
 
         #if self.has_level(LOG_DEBUG):
@@ -391,34 +492,7 @@ class RootDeviceXML(static.Data):
 class MediaServer(log.Loggable,BasicDeviceMixin):
     logCategory = 'mediaserver'
 
-    def __init__(self, coherence, backend, **kwargs):
-        self.coherence = coherence
-        self.device_type = 'MediaServer'
-        self.version = int(kwargs.get('version',self.coherence.config.get('version',2)))
-
-        try:
-            self.uuid = kwargs['uuid']
-        except KeyError:
-            from coherence.upnp.core.uuid import UUID
-            self.uuid = UUID()
-
-        self.backend = None
-        urlbase = self.coherence.urlbase
-        if urlbase[-1] != '/':
-            urlbase += '/'
-        self.urlbase = urlbase + str(self.uuid)[5:]
-
-        self.msg('MediaServer urlbase %s' % self.urlbase)
-
-        kwargs['urlbase'] = self.urlbase
-        self.icons = kwargs.get('iconlist', kwargs.get('icons', []))
-        if len(self.icons) == 0:
-            if kwargs.has_key('icon'):
-                self.icons.append(kwargs['icon'])
-
-        louie.connect( self.init_complete, 'Coherence.UPnP.Backend.init_completed', louie.Any)
-        louie.connect( self.init_failed, 'Coherence.UPnP.Backend.init_failed', louie.Any)
-        reactor.callLater(0.2, self.fire, backend, **kwargs)
+    device_type = 'MediaServer'
 
     def fire(self,backend,**kwargs):
         if kwargs.get('no_thread_needed',False) == False:
@@ -443,13 +517,6 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
         else:
             self.backend = backend(self, **kwargs)
 
-    def init_failed(self, backend, msg):
-        if self.backend != backend:
-            return
-        self.warning('backend not installed, MediaServer activation aborted - %s', msg.getErrorMessage())
-        self.debug(msg)
-        del self.coherence.active_backends[str(self.uuid)]
-
     def init_complete(self, backend):
         if self.backend != backend:
             return
@@ -464,7 +531,10 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
             raise LookupError,msg
 
         try:
-            self.content_directory_server = ContentDirectoryServer(self)
+            transcoding = False
+            if self.coherence.config.get('transcoding', 'no') == 'yes':
+                transcoding = True
+            self.content_directory_server = ContentDirectoryServer(self,transcoding=transcoding)
             self._services.append(self.content_directory_server)
         except LookupError,msg:
             self.warning( 'ContentDirectoryServer', msg)
@@ -509,13 +579,23 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
 
         self.web_resource.putChild('ConnectionManager', self.connection_manager_server)
         self.web_resource.putChild('ContentDirectory', self.content_directory_server)
-        self.web_resource.putChild('X_MS_MediaReceiverRegistrar', self.media_receiver_registrar_server)
+        if hasattr(self,"media_receiver_registrar_server"):
+            self.web_resource.putChild('X_MS_MediaReceiverRegistrar', self.media_receiver_registrar_server)
 
         for icon in self.icons:
             if icon.has_key('url'):
                 if icon['url'].startswith('file://'):
                     self.web_resource.putChild(os.path.basename(icon['url']),
                                                StaticFile(icon['url'][7:]))
+                elif icon['url'] == '.face':
+                    face_path = os.path.join(os.path.expanduser('~'), ".face")
+                    if os.path.exists(face_path):
+                        self.web_resource.putChild('face-icon.png',StaticFile(face_path))
+                else:
+                    from pkg_resources import resource_filename
+                    icon_path = os.path.abspath(resource_filename(__name__, os.path.join('..','..','..','misc','device-icons',icon['url'])))
+                    if os.path.exists(icon_path):
+                        self.web_resource.putChild(icon['url'],StaticFile(icon_path))
 
         self.register()
-        self.warning("%s %s (%s) activated" % (self.backend.name, self.device_type, self.backend))
+        self.warning("%s %s (%s) activated with %s" % (self.backend.name, self.device_type, self.backend, str(self.uuid)[5:]))
